const express = require("express");
const router = express.Router();
const axios = require("axios");
const path = require("path");
const fs = require("fs");
const {
  Patient,
  PatientService,
  PatientTransaction,
  PatientNotification,
} = require("../../models/patient");
const {
  Hospital,
  Service,
  HospitalBank,
  Transaction,
  Notification,
  HospitalNotification,
} = require("../../models/hospital");
const {
  HospitalChatwithAi,
  PatientHospitalChat,
} = require("../../models/chatMessage");
const jwt = require("jsonwebtoken");
const { v4: uuidv4 } = require("uuid");
const sendPasswordResetEmail = require("../../utils/sendEmail");
const verifyToken = require("../../middlewares/verifyToken");
const { hashPassword, comparePasswords } = require("../../helpers/hashPass");
const jwtSecret = process.env.JWT_SECRET_KEY;
const { upload, uploadDestination } = require("../../config/multerConfig");
const africasTalking = require("../../config/africasTalkingConfig");
const { default: mongoose } = require("mongoose");
const generateUniqueTransactionId = require("../../helpers/transId");
const Flutterwave = require("flutterwave-node-v3");
const flw = new Flutterwave(
  process.env.FLW_PUBLIC_KEY,
  process.env.FLW_SECRET_KEY
);

function generateTransactionReference() {
  const uniqueId = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(
    /[xy]/g,
    function (c) {
      const r = (Math.random() * 16) | 0;
      const v = c === "x" ? r : (r & 0x3) | 0x8;
      return v.toString(16);
    }
  );
  const timestamp = Date.now();
  const reference = `${timestamp}-${uniqueId}`;
  return reference;
}

// Function to create a hospital notification
async function createHospitalNotification(
  hospitalId,
  title,
  content,
  transactionId
) {
  try {
    // Check if a HospitalNotification document already exists for the hospitalId
    const existingNotification = await HospitalNotification.findOne({
      hospitalId,
    });

    if (existingNotification) {
      existingNotification.notifications.push({
        title,
        content,
        transactionId,
      });

      await existingNotification.save();
    } else {
      const notification = new HospitalNotification({
        hospitalId,
        notifications: [
          {
            title,
            content,
            transactionId,
          },
        ],
      });

      await notification.save();
    }
  } catch (error) {
    console.error("Error creating hospital notification:", error);
  }
}

// Function to create a hospital notification
async function createPatientNotification(
  patientId,
  title,
  content,
) {
  try {
    // Check if a PatientNotification document already exists for the patientId
    const existingNotification = await PatientNotification.findOne({
      patientId,
    });

    if (existingNotification) {
      existingNotification.notifications.push({
        title,
        content,
      });

      await existingNotification.save();
    } else {
      const notification = new PatientNotification({
        patientId,
        notifications: [
          {
            title,
            content,
          },
        ],
      });

      await notification.save();
    }
  } catch (error) {
    console.error("Error creating hospital notification:", error);
  }
}

// Define an API route for initiating a transfer
router.post("/initiate-transfer", async (req, res) => {
  try {
    const { account_bank, account_number, amount, currency, narration } =
      req.body;

    // Generate a unique reference
    const reference = generateTransactionReference();

    const transferDetails = {
      account_bank,
      account_number,
      amount,
      currency,
      narration,
      reference: reference, // Include the generated reference
    };

    // Initiate the transfer using Flutterwave API
    const transferResult = await flw.Transfer.initiate(transferDetails);

    if (transferResult.status === "success") {
      res.status(200).json({
        transferResult,
      });
    } else {
      // Handle cases where the transfer is not successful
      res.status(500).json({
        error: "Transfer failed",
        transferResult,
      });
    }
  } catch (error) {
    console.error("An error occurred:", error.message);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Define an API route for refunding a payment
router.post("/refund-payment", async (req, res) => {
  try {
    const { transactionId, amount, comments } = req.body;
    const refundResult = await flw.Transaction.refund({
      id: transactionId,
      amount: amount,
      // comments: comments,
    });
    if (refundResult.status === "success") {
      res.status(200).json({
        message: "Payment refunded successfully",
        refundResult,
      });
    } else {
      res.status(500).json({
        error: "Refund failed",
        refundResult,
      });
    }
  } catch (error) {
    console.error("An error occurred:", error.message);
    res.status(500).json({ error: "Internal server error" });
  }
});


async function sendVerificationCode(phoneNumber, verificationCode) {
  africasTalking.SMS.send({
    to: `+${phoneNumber}`,
    message: `Your LetoSave Hospital verification code is ${verificationCode}`,
  })
    .then((res) => console.log("Message send successfully", res))
    .catch((err) => console.log(err));
}

async function sendMessage(phoneNumber, Message) {
  africasTalking.SMS.send({
    to: `${phoneNumber}`,
    message: `${Message}`,
  })
    .then(() => console.log("Message send successfully"))
    .catch((err) => console.log(err));
}

// Register Hospital
router.post(
  "/hospital-register",
  upload.fields([
    { name: "certificationImage", maxCount: 1 },
    { name: "licenseImage", maxCount: 1 },
  ]),
  async (req, res) => {
    try {
      const {
        hospitalName,
        district,
        subCounty,
        village,
        zipCode,
        phone,
        email,
      } = req.body;

      const existingHospital = await Hospital.findOne({
        $or: [{ phone }, { email }],
      });

      if (existingHospital) {
        if (existingHospital.phoneVerified) {
          return res.status(400).json({
            error:
              "Hospital with the provided Phone or Email is already registered",
          });
        }

        const verificationCode = Math.floor(
          1000 + Math.random() * 9000
        ).toString();
        const verificationCodeExpire = new Date(Date.now() + 3600 * 1000);

        existingHospital.hospitalName = hospitalName;
        existingHospital.district = district;
        existingHospital.subCounty = subCounty;
        existingHospital.village = village;
        existingHospital.zipCode = zipCode;
        existingHospital.email = email;
        existingHospital.phone = phone;
        existingHospital.phoneOtp = verificationCode;
        existingHospital.phoneOtpExpire = verificationCodeExpire;

        // Handle certificate image uploads if provided
        if (req.files["certificationImage"]) {
          const certificationImageNewName =
            "uploadscertification-" +
            Date.now() +
            path.extname(req.files["certificationImage"][0].originalname);

          const certificationImagePath =
            uploadDestination + certificationImageNewName;

          fs.renameSync(
            req.files["certificationImage"][0].path,
            certificationImagePath
          );

          existingHospital.certificationImage = certificationImageNewName;
        }

        // Handle license image uploads if provided
        if (req.files["licenseImage"]) {
          const licenseImageNewName =
            "uploadslicense-" +
            Date.now() +
            path.extname(req.files["licenseImage"][0].originalname);

          const licenseImagePath = uploadDestination + licenseImageNewName;

          fs.renameSync(req.files["licenseImage"][0].path, licenseImagePath);

          existingHospital.licenseImage = licenseImageNewName;
        }

        sendVerificationCode(phone, verificationCode);

        await existingHospital.save();

        const payload = { hospitalId: existingHospital._id };
        const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

        return res.status(200).json({
          message: "Verification code sent successfully",
          token,
          hospitalId: existingHospital._id,
          phone,
        });
      } else {
        const verificationCode = Math.floor(
          1000 + Math.random() * 9000
        ).toString();
        const verificationCodeExpire = new Date(Date.now() + 3600 * 1000);

        const certificationImageNewName =
          "uploadscertification-" +
          Date.now() +
          path.extname(req.files["certificationImage"][0].originalname);
        const licenseImageNewName =
          "uploadslicense-" +
          Date.now() +
          path.extname(req.files["licenseImage"][0].originalname);

        const certificationImagePath =
          uploadDestination + certificationImageNewName;
        const licenseImagePath = uploadDestination + licenseImageNewName;

        fs.renameSync(
          req.files["certificationImage"][0].path,
          certificationImagePath
        );
        fs.renameSync(req.files["licenseImage"][0].path, licenseImagePath);

        const newHospital = new Hospital({
          hospitalName,
          district,
          subCounty,
          village,
          zipCode,
          phone,
          email,
          phoneOtp: verificationCode,
          phoneOtpExpire: verificationCodeExpire,
          certificationImage: certificationImageNewName,
          licenseImage: licenseImageNewName,
        });

        await newHospital.save();

        sendVerificationCode(phone, verificationCode);

        const payload = { hospitalId: newHospital._id };
        const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

        return res.status(201).json({
          message: "Verification code sent successfully",
          token,
          hospitalId: newHospital._id,
          phone,
        });
      }
    } catch (error) {
      console.error("Error registering hospital:", error);
      res.status(500).json({ message: "Internal server error" });
    }
  }
);

// Verify Phone OTP
router.post("/hospital-verify-phone-otp", async (req, res) => {
  try {
    const { hospitalId, pin } = req.body;

    const hospital = await Hospital.findById(hospitalId);

    if (!hospital) {
      return res.status(404).json({ message: "Hospital not found" });
    }

    if (hospital.phoneVerified) {
      return res.status(400).json({ message: "Phone already verified" });
    }

    if (hospital.phoneOtpExpire < new Date()) {
      return res.status(400).json({ message: "OTP has expired" });
    }

    if (hospital.phoneOtp !== pin) {
      return res.status(400).json({ message: "Incorrect OTP PIN Code " });
    }

    const forgotToken = uuidv4();
    const oneHourLater = new Date();
    oneHourLater.setHours(oneHourLater.getHours() + 1);

    hospital.forgotToken = forgotToken;
    hospital.forgotTokenExpire = oneHourLater;
    hospital.phoneVerified = true;
    hospital.phoneOtp = null;
    hospital.phoneOtpExpire = null;

    await hospital.save();

    return res
      .status(200)
      .json({ message: "Phone verification successful", forgotToken });
  } catch (error) {
    console.error("Error verifying phone OTP:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Check token
router.get("/hospital-set-password/:id", async (req, res) => {
  try {
    const forgotToken = req.params.id;
    const hospital = await Hospital.findOne({ forgotToken });

    if (!hospital) {
      return res.status(404).json({ message: "Hospital not found" });
    }

    if (!hospital.forgotToken || !hospital.forgotTokenExpire) {
      return res.status(400).json({ message: "Token not found" });
    }

    const tokenExpiration = new Date(hospital.forgotTokenExpire);
    if (tokenExpiration <= new Date()) {
      return res.status(400).json({ message: "Token has expired" });
    }

    res.status(200).json({ message: "Token is valid" });
  } catch (error) {
    console.error("Error checking token validity:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Reset Password first check token
router.post("/hospital-set-password/:id", async (req, res) => {
  try {
    const forgotToken = req.params.id;
    const password = req.body.password;

    const hospital = await Hospital.findOne({ forgotToken });

    if (!hospital) {
      return res.status(404).json({ message: "Hospital not found" });
    }

    if (!hospital.forgotToken || !hospital.forgotTokenExpire) {
      return res.status(400).json({ message: "Token not found" });
    }

    const tokenExpiration = new Date(hospital.forgotTokenExpire);
    if (tokenExpiration <= new Date()) {
      return res.status(400).json({ message: "Token has expired" });
    }

    const hashedPassword = await hashPassword(password);

    hospital.password = hashedPassword;
    hospital.forgotToken = undefined;
    hospital.forgotTokenExpire = undefined;

    await hospital.save();

    const payload = { hospitalId: hospital._id };
    const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

    res.status(200).json({ message: "Password set successful", token });
  } catch (error) {
    console.error("Error resetting password:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Login
router.post("/hospital-login", async (req, res) => {
  try {
    const { email, password } = req.body;

    // Find the hospital using the provided email
    const hospital = await Hospital.findOne({ email });
    if (!hospital) {
      return res.status(401).json({ message: "Invalid credentials" });
    }

    // Check if the user has set a password
    if (!hospital.password) {
      return res.status(400).json({
        message:
          "Password not set. Set your password using the forgot password.",
      });
    }

    // Compare the provided password with the stored password
    const isPasswordValid = await comparePasswords(password, hospital.password);

    if (!isPasswordValid) {
      return res.status(401).json({ message: "Invalid credentials" });
    }

    // Generate a JWT token
    const payload = { hospitalId: hospital._id };
    const jwtSecret = process.env.JWT_SECRET_KEY;
    const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

    res.status(200).json({ message: "Login successful", token });
  } catch (error) {
    console.error("Error during login:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Hospital Details
router.get("/hospital-details", verifyToken, async (req, res) => {
  try {
    const { hospitalId } = req.decodedToken;

    const hospitalDetails = await Hospital.findById(hospitalId)
      .select(
        "_id hospitalName district subCounty village zipCode phone phoneVerified email emailVerified certificationImage licenseImage balance"
      )
      .exec();

    if (!hospitalDetails) {
      return res.status(404).json({ error: "Hospital not found" });
    }

    res.json({ hospitalDetails });
  } catch (error) {
    console.error(error);
    res
      .status(500)
      .json({ error: "An error occurred while fetching hospital details" });
  }
});

// Send Reset Link
router.post("/hospital-forgot-password", async (req, res) => {
  try {
    const { email } = req.body;

    const hospital = await Hospital.findOne({ email });
    if (!hospital) {
      return res.status(404).json({ message: "Hospital not found" });
    }

    const forgotToken = uuidv4();
    const oneHourLater = new Date();
    oneHourLater.setHours(oneHourLater.getHours() + 1);

    hospital.forgotToken = forgotToken;
    hospital.forgotTokenExpire = oneHourLater;
    await hospital.save();

    const resetLink = `${process.env.CLIENT_URI}/create-password/${forgotToken}`;
    await sendPasswordResetEmail(email, resetLink);
    res
      .status(200)
      .json({ message: "Password reset email sent successfully", resetLink });
  } catch (error) {
    console.error("Error sending password reset email:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Change Password
router.post("/hospital-change-password", verifyToken, async (req, res) => {
  try {
    const { currentPassword, newPassword } = req.body;

    const { hospitalId } = req.decodedToken;

    const hospital = await Hospital.findById(hospitalId);
    if (!hospital) {
      return res.status(404).json({ message: "Hospital not found" });
    }

    const isPasswordValid = await comparePasswords(
      currentPassword,
      hospital.password
    );
    if (!isPasswordValid) {
      return res.status(400).json({ message: "Current password is incorrect" });
    }

    const hashedNewPassword = await hashPassword(newPassword);
    hospital.password = hashedNewPassword;
    await hospital.save();

    res.status(200).json({ message: "Password updated successfully" });
  } catch (error) {
    console.error("Error updating password:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Route to get all patients' details
router.get("/patients", async (req, res) => {
  try {
    const patients = await Hospital.find({}, { password: 0 });

    res.status(200).json({ patients });
  } catch (error) {
    console.error("Error fetching patients:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Service Management
// Create a new hospital service
router.post("/hospital-service", async (req, res) => {
  try {
    const {
      hospitalId,
      serviceName,
      wardType,
      roomType,
      deliveryType,
      serviceDescription,
      serviceCost,
      doctorFee,
      includePostnatalCare,
    } = req.body;

    // Check if the hospital with the provided hospitalId exists
    const hospitalExists = await Hospital.exists({ _id: hospitalId });

    if (!hospitalExists) {
      return res
        .status(400)
        .json({ error: "Hospital with the provided ID not found" });
    }

    const service = new Service({
      hospitalId,
      serviceName,
      wardType,
      roomType,
      deliveryType,
      serviceDescription,
      serviceCost,
      doctorFee,
      includePostnatalCare,
    });

    const savedService = await service.save();

    res.status(201).json(savedService);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Get information about all hospital services
router.get("/hospital-service", async (req, res) => {
  try {
    const services = await Service.find().populate(
      "hospitalId",
      "_id hospitalName district subCounty village zipCode phone email certificationImage licenseImage"
    );

    res.status(200).json(services);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Edit Existing Service
router.put("/hospital-service/:serviceId", async (req, res) => {
  try {
    const { serviceId } = req.params;

    const {
      serviceName,
      wardType,
      roomType,
      deliveryType,
      serviceDescription,
      serviceCost,
      doctorFee,
      includePostnatalCare,
    } = req.body;

    const existingService = await Service.findById(serviceId);

    if (!existingService) {
      return res
        .status(404)
        .json({ error: "Service with the provided ID not found" });
    }

    existingService.serviceName = serviceName;
    existingService.wardType = wardType;
    existingService.roomType = roomType;
    existingService.deliveryType = deliveryType;
    existingService.serviceDescription = serviceDescription;
    existingService.serviceCost = serviceCost;
    existingService.doctorFee = doctorFee;
    existingService.includePostnatalCare = includePostnatalCare;

    const updatedService = await existingService.save();

    res.status(200).json(updatedService);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Delete a specific hospital service by serviceId
router.delete("/hospital-service/:serviceId", async (req, res) => {
  try {
    const { serviceId } = req.params;
    const existingService = await Service.findById(serviceId);
    if (!existingService) {
      return res
        .status(404)
        .json({ error: "Service with the provided ID not found" });
    }
    await existingService.deleteOne();
    res.status(204).json();
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Get information about a specific hospital service by serviceId
router.get("/hospital-service/:serviceId", async (req, res) => {
  try {
    const { serviceId } = req.params;

    const service = await Service.findById(serviceId);

    if (!service) {
      return res
        .status(404)
        .json({ error: "Service with the provided ID not found" });
    }

    res.status(200).json(service);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Get all services related to a specific hospital by hospitalId
router.get("/hospital-services/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    const services = await Service.find({ hospitalId });

    res.status(200).json(services);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Define a route to get related hospital services without verifyToken
router.post("/related-hospital-services", async (req, res) => {
  try {
    const hospitalId = req.body.hospitalId;

    if (!hospitalId) {
      return res
        .status(400)
        .json({ error: "Hospital ID not provided in the request body" });
    }

    // Check if the hospital exists and is valid
    const hospital = await Hospital.findById(hospitalId);

    if (!hospital) {
      return res.status(404).json({ error: "Hospital not found or not valid" });
    }

    const patientServices = await PatientService.find({
      hospitalId: hospitalId,
    })
      .populate("service")
      .populate(
        "patient",
        "_id name email phone dateOfBirth expectedDelivery childsGender profilePicture"
      );

    res.status(200).json({ patientServices });
  } catch (error) {
    console.error("Error fetching related hospital services:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Define a route to get details of a single patientService by ID
router.get("/patient-service/:id", verifyToken, async (req, res) => {
  try {
    const patientServiceId = req.params.id;

    if (!mongoose.Types.ObjectId.isValid(patientServiceId)) {
      return res.status(400).json({ error: "Invalid patientService ID" });
    }

    const patientService = await PatientService.findById(patientServiceId)
      .populate("service")
      .populate(
        "patient",
        "_id name email phone dateOfBirth expectedDelivery childsGender profilePicture"
      );

    if (!patientService) {
      return res.status(404).json({ error: "PatientService not found" });
    }

    const patientServiceTransations = await PatientTransaction.find({
      patientServiceId,
    }).populate("patientId", "_id name phone");
    if (!patientServiceTransations) {
      return res
        .status(200)
        .json({ patientService, patientServiceTransations: "not yet" });
    }

    res.status(200).json({ patientService, patientServiceTransations });
  } catch (error) {
    console.error("Error fetching patientService details:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Create a new route to accept or decline a service
router.put("/accept-decline-service", verifyToken, async (req, res) => {
  try {
    const { patientServiceId, status } = req.body;

    if (!mongoose.Types.ObjectId.isValid(patientServiceId)) {
      return res.status(400).json({ error: "Invalid patientService ID" });
    }

    const patientService = await PatientService.findById(patientServiceId)
      .populate('service') 
      .populate('hospitalId'); 

    if (!patientService) {
      return res.status(404).json({ error: "PatientService not found" });
    }

    if (patientService.status === "pending") {
      patientService.status = status;

      patientService.timeResponse = new Date();

      let notificationHeading = ""; 
      let notificationContent = ""; 

      if (status === "approved") {
        notificationHeading = "Service Approved"; 
        notificationContent = `Your service "${patientService.service.serviceName}" has been approved by ${patientService.hospitalId.hospitalName}.`; 
      } else if (status === "declined") {
        notificationHeading = "Service Declined"; 
        notificationContent = `Your service "${patientService.service.serviceName}" has been declined by ${patientService.hospitalId.hospitalName}.`; 
      }

      createPatientNotification(
        patientService.patient,
        notificationHeading, 
        notificationContent, 
      );


      await patientService.save();

      return res
        .status(200)
        .json({ message: `Service ${status.toUpperCase()}` });
    } else {
      return res
        .status(400)
        .json({ error: "Service is not pending and cannot be updated" });
    }
  } catch (error) {
    console.error("Error accepting/declining service:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Chat Management
// Define the route to send a message
router.post("/send-message", async (req, res) => {
  try {
    const { phoneNumber, message } = req.body;

    await sendMessage(phoneNumber, message);

    res.status(200).json({ message: "Message sent successfully" });
  } catch (err) {
    console.error("Error sending message:", err);
    res
      .status(500)
      .json({ error: "An error occurred while sending the message" });
  }
});

// Create a new chat session or add a message to an existing session
router.post("/save-chat", async (req, res) => {
  try {
    const { hospitalId, content, role, createNewSession } = req.body;

    // Find the hospital chat document associated with the hospitalId
    let chat = await HospitalChatwithAi.findOne({ hospitalId });

    if (!chat) {
      // If the chat document doesn't exist, create a new one with a session
      chat = new HospitalChatwithAi({
        hospitalId,
        chatSessions: [{ chatHistory: [] }],
      });
    }

    // Use the last session or create a new one if requested
    const session = createNewSession
      ? { chatHistory: [] } // Create a new session
      : chat.chatSessions[chat.chatSessions.length - 1];

    session.chatHistory.push({
      time: new Date(),
      content,
      role,
    });

    if (createNewSession) {
      chat.chatSessions.push(session); // Add the new session to chatSessions
    }

    await chat.save();

    res.status(201).json({ message: "Chat message saved successfully" });
  } catch (error) {
    console.error("Error saving chat message:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Create a new chat session
router.post("/create-chat-session", async (req, res) => {
  try {
    const { hospitalId } = req.body;

    const chatSession = {
      chatHistory: [],
    };

    const existingChat = await HospitalChatwithAi.findOne({ hospitalId });

    if (!existingChat) {
      const chat = new HospitalChatwithAi({
        hospitalId,
        chatSessions: [chatSession],
      });
      await chat.save();
    } else {
      existingChat.chatSessions.push(chatSession);
      await existingChat.save();
    }

    res.status(201).json({ message: "New chat session created successfully" });
  } catch (error) {
    console.error("Error creating chat session:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Add a new route to get all chat history for the last session
router.get("/get-last-session-chat-history/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    const chat = await HospitalChatwithAi.findOne({ hospitalId });

    if (!chat) {
      return res.status(404).json({ message: "Chat history not found" });
    }

    const lastSession =
      chat.chatSessions.length > 0
        ? chat.chatSessions[chat.chatSessions.length - 1]
        : null;

    if (!lastSession) {
      return res.status(404).json({ message: "Last session not found" });
    }

    const chatHistory = lastSession.chatHistory;

    res.status(200).json({ chatHistory });
  } catch (error) {
    console.error("Error getting last session chat history:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Create a new message between sender and receiver
router.post("/patient-hospital-chat", async (req, res) => {
  try {
    const { sender, receiver, content } = req.body;

    if (
      !mongoose.isValidObjectId(sender) ||
      !mongoose.isValidObjectId(receiver)
    ) {
      return res
        .status(400)
        .json({ error: "Invalid ObjectId for sender or receiver" });
    }

    const message = new PatientHospitalChat({
      sender,
      receiver,
      content,
    });

    await message.save();

    res.status(201).json({ message: "Message sent successfully", message });
  } catch (error) {
    console.error("Error sending message:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// API to retrieve all chats for a given user
router.get("/active-chats/:userId", async (req, res) => {
  try {
    const userId = req.params.userId;

    // Find all messages where the user is the sender or receiver
    const chats = await PatientHospitalChat.find({
      $or: [{ "sender.id": userId }, { "receiver.id": userId }],
    });

    // Fetch user details for sender and receiver
    const chatDetails = await Promise.all(
      chats.map(async (chat) => {
        const senderDetails =
          chat.sender.type === "Hospital"
            ? await Hospital.findById(chat.sender.id).select("-password")
            : await Patient.findById(chat.sender.id).select("-password");

        const receiverDetails =
          chat.receiver.type === "Hospital"
            ? await Hospital.findById(chat.receiver.id).select("-password")
            : await Patient.findById(chat.receiver.id).select("-password");

        return {
          ...chat._doc,
          isSender: chat.sender.id.toString() === userId,
          senderDetails,
          receiverDetails,
        };
      })
    );

    res.status(200).json({ chats: chatDetails });
  } catch (error) {
    console.error("Error retrieving chats:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// API to retrieve all chats between two users
router.get("/chats/:userId1/:userId2", async (req, res) => {
  try {
    const userId1 = req.params.userId1;
    const userId2 = req.params.userId2;

    // Find all messages exchanged between the two users
    const chats = await PatientHospitalChat.find({
      $or: [
        { "sender.id": userId1, "receiver.id": userId2 },
        { "sender.id": userId2, "receiver.id": userId1 },
      ],
    });

    // Fetch user details for sender and receiver
    const chatDetails = await Promise.all(
      chats.map(async (chat) => {
        const senderDetails =
          chat.sender.type === "Hospital"
            ? await Hospital.findById(chat.sender.id).select("-password")
            : await Patient.findById(chat.sender.id).select("-password");

        const receiverDetails =
          chat.receiver.type === "Hospital"
            ? await Hospital.findById(chat.receiver.id).select("-password")
            : await Patient.findById(chat.receiver.id).select("-password");

        return {
          ...chat._doc,
          isSender: chat.sender.id.toString() === userId1,
          senderDetails,
          receiverDetails,
        };
      })
    );

    res.status(200).json({ chats: chatDetails });
  } catch (error) {
    console.error("Error retrieving chats:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Bank Routes for hospital
// Add a new hospital bank
router.post("/add-hospital-bank", async (req, res) => {
  try {
    const {
      hospitalId,
      accountHoldersName,
      bankName,
      bankBranch,
      accountNumber,
      bankSwiftCode,
      phoneNumber,
      status,
    } = req.body;

    const hospitalExists = await Hospital.exists({ _id: hospitalId });

    if (!hospitalExists) {
      return res
        .status(400)
        .json({ error: "Hospital with the provided ID not found" });
    }

    const hospitalBank = new HospitalBank({
      hospitalId,
      accountHoldersName,
      bankName,
      bankBranch,
      accountNumber,
      bankSwiftCode,
      phoneNumber,
      status,
    });

    const savedHospitalBank = await hospitalBank.save();

    res.status(201).json(savedHospitalBank);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Retrieve all bank details for a hospital
router.get("/get-hospital-banks", async (req, res) => {
  try {
    const { hospitalId } = req.query;
    const hospitalExists = await Hospital.exists({ _id: hospitalId });

    if (!hospitalExists) {
      return res
        .status(400)
        .json({ error: "Hospital with the provided ID not found" });
    }

    const bankDetails = await HospitalBank.find({ hospitalId });

    res.status(200).json(bankDetails);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

router.post("/get-hospital-banks-transactions", async (req, res) => {
  try {
    const { hospitalId } = req.body;
    const hospitalExists = await Hospital.exists({ _id: hospitalId });

    if (!hospitalExists) {
      return res
        .status(400)
        .json({ error: "Hospital with the provided ID not found" });
    }

    // Find all bank details for the given hospital
    const bankDetails = await HospitalBank.find({ hospitalId });

    // Populate all transactions for each bank
    const banksWithTransactions = await Promise.all(
      bankDetails.map(async (bank) => {
        const transactions = await Transaction.find({
          hospitalId,
          hospitalBankId: bank._id,
        });

        // Filter approved transactions
        const approvedTransactions = transactions.filter(
          (transaction) => transaction.status === "approved"
        );

        // Calculate the totalWithdraw based on approved transactions
        const totalWithdraw = approvedTransactions.reduce(
          (acc, transaction) => acc + transaction.amount,
          0
        );

        return {
          ...bank.toObject(),
          transactions,
          totalWithdraw,
        };
      })
    );

    res.status(200).json(banksWithTransactions);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Create a new hospital transaction
router.post("/create-hospital-transaction", async (req, res) => {
  try {
    const { hospitalId, hospitalBankId, amount } = req.body;

    const hospital = await Hospital.findById(hospitalId);

    if (!hospital) {
      return res.status(400).json({ error: "Hospital not found" });
    }

    const hospitalBank = await HospitalBank.findById(hospitalBankId);

    if (!hospitalBank) {
      return res.status(400).json({ error: "Hospital Bank not found" });
    }

    if (amount > hospital.balance) {
      return res.status(400).json({ error: "Insufficient balance" });
    }

    const transId = await generateUniqueTransactionId(hospitalId);

    // Get the transfer fee from Flutterwave
    const feeQueryParams = {
      amount: amount,
      currency: hospitalBank.bankAccountCurrency,
    };

    let transferFeeResponse;

    try {
      const feeResponse = await axios.get(
        "https://api.flutterwave.com/v3/transfers/fee",
        {
          params: feeQueryParams,
          headers: {
            Authorization: `Bearer ${process.env.FLW_SECRET_KEY}`,
          },
        }
      );
      transferFeeResponse = feeResponse.data.data;
      // console.log(transferFeeResponse);
    } catch (error) {
      transferFeeResponse = error.response.data.data;
    }

    // Calculate the amount after deducting the fee
    const amountAfterFee = amount - transferFeeResponse[0].fee;

    // Create a new Transaction document
    const transaction = new Transaction({
      hospitalId,
      hospitalBankId,
      amount: amount, // Save the amount after deducting the fee
      amountBeforeFee: amount, // Save the full amount before the fee deduction
      transDate: new Date(),
      transId,
      hospitalBalance: hospital.balance,
      transferFeeResponse,
    });

    // Save the transaction to the database
    await transaction.save();

    // Update the hospital's balance by subtracting the amount
    hospital.balance -= amount;

    // Save the updated hospital object
    await hospital.save();

    res.status(201).json(transaction);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Route to get all transactions and hospital details by hospitalId
router.get("/hospital-transactions/:hospitalId", async (req, res) => {
  try {
    const hospitalId = req.params.hospitalId;

    const hospital = await Hospital.findById(hospitalId);

    if (!hospital) {
      return res.status(404).json({ error: "Hospital not found" });
    }

    const transactions = await Transaction.find({ hospitalId }).populate(
      "hospitalBankId",
      "accountHoldersName bankName bankBranch accountNumber bankSwiftCode phoneNumber status"
    );

    const response = {
      // hospital: {
      // _id: hospital._id,
      // hospitalName: hospital.hospitalName,
      // email: hospital.email,
      // },
      transactions: transactions.map((transaction) => ({
        _id: transaction._id,
        transDate: transaction.transDate,
        transId: transaction.transId,
        amount: transaction.amount,
        status: transaction.status,
        hospitalBank: transaction.hospitalBankId,
        hospitalBalance: transaction.hospitalBalance,
      })),
    };

    res.status(200).json(response);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Route to get all patient transactions related to a specific hospitalId
router.get("/hospital-patient-transactions/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    const patientTransactions = await PatientTransaction.find({ hospitalId })
      .populate(
        "patientId",
        "_id name email phone profilePicture balance patientServiceId expectedDelivery"
      )
      .populate(
        "hospitalBankId",
        "_id accountHoldersName bankName bankBranch accountNumber bankSwiftCode phoneNumber status"
      )
      .populate("serviceId", "_id serviceName time serviceCost")
      .populate("patientServiceId", "_id reBalance amountSaved time");

    if (!patientTransactions) {
      return res.status(404).json({ error: "No patient transactions found" });
    }

    res.status(200).json(patientTransactions);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Route to get all patient transactions related to a specific hospitalId and patientId
router.post("/hospital-patient-transactions", async (req, res) => {
  try {
    const { hospitalId, patientId } = req.body;

    const patientTransactions = await PatientTransaction.find({
      hospitalId,
      patientId,
    })
      .populate("patientId", "_id name email phone profilePicture balance")
      .populate(
        "hospitalBankId",
        "_id accountHoldersName bankName bankBranch accountNumber bankSwiftCode phoneNumber status"
      )
      .populate("patientServiceId", "_id reBalance amountSaved");

    if (!patientTransactions) {
      return res.status(404).json({ error: "No patient transactions found" });
    }

    res.status(200).json(patientTransactions);
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Route to get total revenue, total deposit, expected deliveries, total deliveries done, payment distribution,
// past 12 months patient services, service names with quantities, and the latest 5 minimum reBalance services
router.get("/hospital-revenue-deposit/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;
    const patientServices = await PatientService.find({ hospitalId })
      .populate("service")
      .populate("patient");

    if (patientServices.length === 0) {
      return res
        .status(404)
        .json({ error: "No patient services transactions found" });
    }

    const uniquePatientIds = new Set();
    let totalRevenue = 0;
    let totalDeposit = 0;
    let expectedDeliveries = 0;
    let totalDeliveriesDone = 0;

    const patients = [];
    const upcomingDeliveries = new Array(9).fill(0);
    const now = new Date();
    const paymentByPercentage = [0, 0, 0, 0, 0];
    const patientsByAge = [0, 0, 0, 0, 0];
    const servicesWithQuantities = new Map();
    const revenueByServiceRoomType = new Map();
    const revenueByWardRoomDelivery = new Map();
    const past12MonthsServices = new Array(12).fill(0);
    const past12MonthsServiceRevenue = new Array(12).fill(0);

    for (const patientService of patientServices) {
      const { patient, status, reBalance, amountSaved, time, service } =
        patientService;
      const serviceName = service.serviceName;
      const roomType = service.roomType;
      const wardType = service.wardType;
      const deliveryType = service.deliveryType;

      let patientDetails;

      if (status === "approved" && !uniquePatientIds.has(patient.toString())) {
        uniquePatientIds.add(patient.toString());

        patientDetails = await Patient.findById(patient).select(
          "_id expectedDelivery dateOfBirth"
        );

        if (patientDetails) {
          patients.push(patientDetails);

          const expectedDeliveryDate = new Date(
            patientDetails.expectedDelivery
          );

          if (expectedDeliveryDate > now) {
            expectedDeliveries++;
            const monthDifference =
              expectedDeliveryDate.getMonth() - now.getMonth();
            if (monthDifference >= 0 && monthDifference < 9) {
              upcomingDeliveries[monthDifference]++;
            }
          } else {
            totalDeliveriesDone++;
          }

          const dateOfBirth = new Date(patientDetails.dateOfBirth);
          const age = now.getFullYear() - dateOfBirth.getFullYear();

          if (age >= 18 && age <= 24) {
            patientsByAge[0]++;
          } else if (age >= 25 && age <= 32) {
            patientsByAge[1]++;
          } else if (age >= 33 && age <= 40) {
            patientsByAge[2]++;
          } else if (age >= 41 && age <= 50) {
            patientsByAge[3]++;
          } else {
            patientsByAge[4]++;
          }
        }
      }

      totalDeposit += amountSaved;

      if (status === "approved") {
        totalRevenue += amountSaved;
        totalRevenue += reBalance;

        const totalBalance = amountSaved + reBalance;
        const paidPercentage = (amountSaved / totalBalance) * 100;

        if (paidPercentage <= 25) {
          paymentByPercentage[0]++;
        } else if (paidPercentage <= 50) {
          paymentByPercentage[1]++;
        } else if (paidPercentage <= 75) {
          paymentByPercentage[2]++;
        } else {
          paymentByPercentage[3]++;
        }

        if (reBalance === 0) {
          paymentByPercentage[4]++;
        }
      }

      const serviceMonthIndex = new Date(time).getMonth();
      const currentMonthIndex = now.getMonth();
      const monthsAgo = currentMonthIndex - serviceMonthIndex;

      if (monthsAgo >= 0 && monthsAgo <= 11) {
        past12MonthsServices[11 - monthsAgo]++;
        past12MonthsServiceRevenue[11 - monthsAgo] += amountSaved;
      }

      // Count services and their quantities
      if (servicesWithQuantities.has(serviceName)) {
        servicesWithQuantities.set(serviceName, {
          count: servicesWithQuantities.get(serviceName).count + 1,
          totalAmountSaved:
            servicesWithQuantities.get(serviceName).totalAmountSaved +
            amountSaved,
        });
      } else {
        servicesWithQuantities.set(serviceName, {
          count: 1,
          totalAmountSaved: amountSaved,
        });
      }

      // Calculate revenue by service roomType
      const serviceRoomKey = `${serviceName}-${roomType}`;
      if (revenueByServiceRoomType.has(serviceRoomKey)) {
        revenueByServiceRoomType.set(
          serviceRoomKey,
          revenueByServiceRoomType.get(serviceRoomKey) + amountSaved
        );
      } else {
        revenueByServiceRoomType.set(serviceRoomKey, amountSaved);
      }

      // Calculate revenue by wardType, roomType, and deliveryType
      const wardRoomDeliveryKey = `${wardType}-${roomType}-${deliveryType}`;
      if (revenueByWardRoomDelivery.has(wardRoomDeliveryKey)) {
        revenueByWardRoomDelivery.set(
          wardRoomDeliveryKey,
          revenueByWardRoomDelivery.get(wardRoomDeliveryKey) + amountSaved
        );
      } else {
        revenueByWardRoomDelivery.set(wardRoomDeliveryKey, amountSaved);
      }
    }

    const servicesWithQuantitiesArray = Array.from(
      servicesWithQuantities.entries()
    ).map(([serviceName, data]) => ({
      serviceName,
      count: data.count,
      totalAmountSaved: data.totalAmountSaved,
    }));

    const revenueByServiceRoomTypeArray = Array.from(
      revenueByServiceRoomType.entries()
    ).map(([serviceRoomKey, revenue]) => {
      const [serviceName, roomType] = serviceRoomKey.split("-");
      return {
        serviceName,
        roomType,
        revenue,
      };
    });

    const revenueByWardRoomDeliveryArray = Array.from(
      revenueByWardRoomDelivery.entries()
    ).map(([wardRoomDeliveryKey, revenue]) => {
      const [wardType, roomType, deliveryType] = wardRoomDeliveryKey.split("-");
      return {
        wardType,
        roomType,
        deliveryType,
        revenue,
      };
    });

    paymentByPercentage.reverse();

    res.status(200).json({
      totalRevenue,
      totalDeposit,
      expectedDeliveries,
      totalDeliveriesDone,
      paymentByPercentage,
      patientsByAge,
      patients,
      upcomingDeliveries,
      past12MonthsServices,
      past12MonthsServiceRevenue,
      servicesWithQuantities: servicesWithQuantitiesArray,
      revenueByServiceRoomType: revenueByServiceRoomTypeArray,
      revenueByWardRoomDelivery: revenueByWardRoomDeliveryArray,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Route to get service-wise revenue data for the previous 12 months
router.get(
  "/hospital-services-revenue-months/:hospitalId",
  async (req, res) => {
    try {
      const { hospitalId } = req.params;

      // Get the current date
      const currentDate = new Date();

      // Calculate the start date for 12 months ago
      const startDate = new Date();
      startDate.setMonth(currentDate.getMonth() - 11);

      const patientTransactions = await PatientTransaction.find({ hospitalId })
        .populate("serviceId", "_id serviceName serviceCost")
        .populate("patientServiceId", "_id time amountSaved status");

      if (!patientTransactions) {
        return res.status(404).json({ error: "No patient transactions found" });
      }

      // Extract unique service names
      const serviceNames = new Set();
      patientTransactions.forEach((transaction) => {
        const { serviceName } = transaction.serviceId;
        serviceNames.add(serviceName);
      });

      // Create an array of month-year strings for the past 12 months with abbreviated month names
      const monthYearStrings = [];
      let currentMonth = new Date(startDate);
      for (let i = 0; i < 12; i++) {
        const monthName = currentMonth.toLocaleString("default", {
          month: "short",
        });
        monthYearStrings.push(monthName); // Remove year from the string
        currentMonth.setMonth(currentMonth.getMonth() + 1);
      }

      // Initialize the service revenue data structure
      const serviceRevenueData = Array.from(serviceNames).map(
        (serviceName) => ({
          serviceName,
          data: monthYearStrings.map((monthName) => ({
            // Use monthName instead of monthYear
            monthName,
            totalAmount: 0,
          })),
        })
      );

      // Populate the revenue data
      patientTransactions.forEach((transaction) => {
        const transactionDate = new Date(transaction.patientServiceId.time);

        // Check if the transaction date is within the past 12 months and the transaction is approved
        if (
          transactionDate >= startDate &&
          transactionDate <= currentDate &&
          transaction.patientServiceId.status === "approved"
        ) {
          const monthName = transactionDate.toLocaleString("default", {
            month: "short",
          });
          const { serviceName, serviceCost } = transaction.serviceId;

          // Find the corresponding data entry and increment the total amount
          const entry = serviceRevenueData.find(
            (data) => data.serviceName === serviceName
          );
          if (entry) {
            const dataEntry = entry.data.find(
              (entry) => entry.monthName === monthName
            );
            if (dataEntry) {
              dataEntry.totalAmount += serviceCost;
            }
          }
        }
      });

      res.status(200).json({
        serviceRevenueData,
      });
    } catch (error) {
      console.error(error);
      res.status(500).json({ error: "Internal Server Error" });
    }
  }
);

router.get("/hospital-revenue-days/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    // Calculate the date 7 days ago from today
    const sevenDaysAgo = new Date();
    sevenDaysAgo.setDate(sevenDaysAgo.getDate() - 7);

    const patientTransactions = await PatientTransaction.find({
      hospitalId,
      time: { $gte: sevenDaysAgo }, // Filter transactions from the past 7 days
    });

    if (patientTransactions.length === 0) {
      return res
        .status(404)
        .json({ error: "No patient transactions found in the past 7 days" });
    }

    // Define an array to map day indices to their short names
    const daysOfWeek = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];

    // Initialize an array to store transaction counts for the previous 7 days
    const previousDaysServiceCounts = new Array(7).fill(0);

    const today = new Date().getDay();

    for (const transaction of patientTransactions) {
      // Calculate the day index for the transaction
      const transactionDate = new Date(transaction.time);
      const transactionDay = transactionDate.getDay();

      // Calculate the number of days ago for the transaction
      const daysAgo = (today - transactionDay + 7) % 7;

      if (daysAgo >= 0 && daysAgo < 7) {
        previousDaysServiceCounts[daysAgo]++;
      }
    }

    // Calculate the previous 7 days' date strings in chronological order
    const previousDays = [];
    for (let i = 0; i < 7; i++) {
      const dayIndex = (today - i + 7) % 7;
      const shortDayName = daysOfWeek[dayIndex];
      previousDays.push(shortDayName);
    }

    res.status(200).json({
      previousDaysServiceCounts,
      previousDays,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Notification Management Hospital
// Route to get all notifications by hospitalId with populated details
router.get("/hospital-notifications/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;
    const hospitalNotifications = await HospitalNotification.findOne({
      hospitalId: hospitalId,
    })
      .populate({
        path: "notifications.serviceId",
        model: "Service",
        select: "serviceName serviceCost",
      })
      .populate({
        path: "notifications.transactionId",
        model: "PatientTransaction",
        select: "transId amount",
      });
    if (!hospitalNotifications) {
      return res
        .status(404)
        .json({ message: "Hospital notifications not found" });
    }
    const notifications = hospitalNotifications.notifications;
    res.status(200).json({ notifications });
  } catch (error) {
    console.error("Error fetching hospital notifications:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Route to update isRead status by hospitalId for notifications with isRead set to false
router.put("/update-is-read/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    const result = await HospitalNotification.updateMany(
      { hospitalId: hospitalId, "notifications.isRead": false },
      {
        $set: {
          "notifications.$[elem].isRead": true,
          "notifications.$[elem].isReadTime": new Date(),
        },
      },
      { arrayFilters: [{ "elem.isRead": false }] }
    );

    if (result.nModified === 0) {
      return res
        .status(404)
        .json({ message: "No unread notifications found to update" });
    }

    res.status(200).json({ message: "isRead status updated successfully" });
  } catch (error) {
    console.error("Error updating isRead status:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Route to update isRemove status by hospitalId based on isRead and isRemove status
router.put("/update-is-remove/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;
    const resultIsRead = await HospitalNotification.updateMany(
      {
        hospitalId: hospitalId,
        "notifications.isRead": false,
      },
      {
        $set: {
          "notifications.$[elem].isRead": true,
          "notifications.$[elem].isReadTime": new Date(),
        },
      },
      {
        arrayFilters: [{ "elem.isRead": false }],
      }
    );
    const resultIsRemove = await HospitalNotification.updateMany(
      {
        hospitalId: hospitalId,
        "notifications.isRemove": false,
      },
      {
        $set: {
          "notifications.$[elem].isRemove": true,
          "notifications.$[elem].isRemoveTime": new Date(),
        },
      },
      {
        arrayFilters: [{ "elem.isRemove": false }],
      }
    );
    if (
      (resultIsRead.nModified === 0 && resultIsRemove.nModified === 0) ||
      (resultIsRead.ok !== 1 && resultIsRemove.ok !== 1)
    ) {
      return res.status(404).json({ message: "Update notifications success" });
    }
    res.status(200).json({ message: "isRemove status updated successfully" });
  } catch (error) {
    console.error("Error updating isRemove status:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Route to update a single notification's isRead status by hospitalId and notificationId
router.put("/update-is-read/:hospitalId/:notificationId", async (req, res) => {
  try {
    const { hospitalId, notificationId } = req.params;

    const hospitalNotification = await HospitalNotification.findOne({
      hospitalId: hospitalId,
      "notifications._id": notificationId,
    });

    if (!hospitalNotification) {
      return res.status(404).json({ message: "Notification not found" });
    }

    const notification = hospitalNotification.notifications.find(
      (notif) => notif._id.toString() === notificationId
    );

    if (!notification) {
      return res.status(404).json({ message: "Notification not found" });
    }

    if (notification.isRead) {
      return res.status(200).json({ message: "Notification already updated" });
    }

    const result = await HospitalNotification.updateOne(
      {
        hospitalId: hospitalId,
        "notifications._id": notificationId,
      },
      {
        $set: {
          "notifications.$.isRead": true,
          "notifications.$.isReadTime": new Date(),
        },
      }
    );

    if (result.nModified === 0) {
      return res.status(200).json({ message: "Notification already updated" });
    }

    res.status(200).json({ message: "isRead status updated successfully" });
  } catch (error) {
    console.error("Error updating isRead status:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Route to update a single notification's isRemove status by hospitalId and notificationId
router.put("/update-is-remove/:hospitalId/:notificationId", async (req, res) => {
    try {
      const { hospitalId, notificationId } = req.params;

      const hospitalNotification = await HospitalNotification.findOne({
        hospitalId: hospitalId,
        "notifications._id": notificationId,
      });

      if (!hospitalNotification) {
        return res.status(404).json({ message: "Notification not found" });
      }

      const notification = hospitalNotification.notifications.find(
        (notif) => notif._id.toString() === notificationId
      );

      if (!notification) {
        return res.status(404).json({ message: "Notification not found" });
      }

      if (notification.isRemove) {
        return res
          .status(200)
          .json({ message: "Notification already updated" });
      }

      const updateQuery = {
        $set: {
          "notifications.$.isRemove": true,
          "notifications.$.isRemoveTime": new Date(),
        },
      };

      if (!notification.isRead) {
        updateQuery.$set["notifications.$.isRead"] = true;
        updateQuery.$set["notifications.$.isReadTime"] = new Date();
      }

      const result = await HospitalNotification.updateOne(
        {
          hospitalId: hospitalId,
          "notifications._id": notificationId,
        },
        updateQuery
      );

      if (result.nModified === 0) {
        return res
          .status(200)
          .json({ message: "Notification already updated" });
      }

      res.status(200).json({ message: "isRemove status updated successfully" });
    } catch (error) {
      console.error("Error updating isRemove status:", error);
      res.status(500).json({ error: "Internal server error" });
    }
  }
);



const FLUTTERWAVE_BASE_URL = "https://api.flutterwave.com/v3";

// for Banks

router.get("/bank-select/:id", async (req, res) => {
  try {
    const bankId = req.params.id;
    const secretKey = process.env.FLW_SECRET_KEY;
    const headers = {
      Authorization: `Bearer ${secretKey}`,
    };
    const response = await axios.get(
      `${FLUTTERWAVE_BASE_URL}/banks/${bankId}`,
      {
        headers: headers,
      }
    );
    res.json(response.data);
  } catch (error) {
    console.error("Error fetching bank details:", error);
    res.status(500).json({ error: "Failed to fetch bank details" });
  }
});

router.get("/bank-select/branches/:id", async (req, res) => {
  try {
    const bankId = req.params.id;
    const secretKey = process.env.FLW_SECRET_KEY;
    const headers = {
      Authorization: `Bearer ${secretKey}`,
    };
    const response = await axios.get(
      `${FLUTTERWAVE_BASE_URL}/banks/${bankId}/branches`,
      {
        headers: headers,
      }
    );
    res.json(response.data);
  } catch (error) {
    console.error("Error fetching bank branches:", error);
    res.status(500).json({ error: "Failed to fetch bank branches" });
  }
});

// router.post("/save-bank", async (req, res) => {
//   try {
//     const {
//       hospitalId,
//       accountHolderName,
//       country,
//       selectedBankName,
//       selectedBankBranch,
//       phoneNumber,
//       accountNumber,
//       swiftCode,
//       bankAccountCurrency,
//       isPrimary,
//     } = req.body;

//     const hospital = await Hospital.findById(hospitalId).select(
//       "hospitalName _id email"
//     );
//     if (!hospital) {
//       return res
//         .status(404)
//         .json({ status: "error", message: "Hospital not found" });
//     }

//     let flutterwaveResponseData;

//     try {
//       const flutterwaveResponse = await axios.post(
//         "https://api.flutterwave.com/v3/subaccounts",
//         {
//           account_bank: selectedBankName,
//           account_number: accountNumber,
//           business_name: accountHolderName,
//           business_email: hospital.email,
//           country: country,
//           split_type: "percentage",
//           split_value: process.env.FLW_COMMISSIOM,
//         },
//         {
//           headers: {
//             Authorization: `Bearer ${process.env.FLW_SECRET_KEY}`,
//           },
//         }
//       );

//       flutterwaveResponseData = flutterwaveResponse.data;

//       const newHospitalBank = new HospitalBank({
//         hospitalId,
//         accountHoldersName: accountHolderName,
//         bankName: selectedBankName,
//         bankBranch: selectedBankBranch,
//         accountNumber,
//         bankSwiftCode: swiftCode,
//         phoneNumber,
//         bankCountry: country,
//         bankAccountCurrency,
//         isPrimary,
//         flutterwaveResponseData: flutterwaveResponseData,
//       });

//       if (flutterwaveResponseData && flutterwaveResponseData.status === 'success') {
//         newHospitalBank.status = 'pending';
//       } else {
//         newHospitalBank.status = 'declined';
//       }

//       await newHospitalBank.save();

//       const responseData = {
//         hospitalId,
//         accountHolderName,
//         country,
//         selectedBankName,
//         selectedBankBranch,
//         phoneNumber,
//         accountNumber,
//         swiftCode,
//         bankAccountCurrency,
//         isPrimary,
//         hospitalBankId: newHospitalBank._id,
//         flutterwaveResponseData: flutterwaveResponseData,
//       };

//       return res.status(200).json({ status: "success", data: responseData, hospital });
//     } catch (flutterwaveError) {
//       console.error("Error from Flutterwave API:", flutterwaveError.response.data);

//       const newHospitalBank = new HospitalBank({
//         hospitalId,
//         accountHoldersName: accountHolderName,
//         bankName: selectedBankName,
//         bankBranch: selectedBankBranch,
//         accountNumber,
//         bankSwiftCode: swiftCode,
//         phoneNumber,
//         bankCountry: country,
//         bankAccountCurrency,
//         isPrimary,
//         flutterwaveResponseData: flutterwaveError.response.data || '',
//       });

//       if (flutterwaveResponseData && flutterwaveResponseData.status === 'success') {
//         newHospitalBank.status = 'pending';
//       } else {
//         newHospitalBank.status = 'declined';
//       }

//       await newHospitalBank.save();

//       const responseData = {
//         hospitalId,
//         accountHolderName,
//         country,
//         selectedBankName,
//         selectedBankBranch,
//         phoneNumber,
//         accountNumber,
//         swiftCode,
//         bankAccountCurrency,
//         isPrimary,
//         hospitalBankId: newHospitalBank._id,
//         flutterwaveResponseData: flutterwaveError.response.data,
//       };

//       return res.status(200).json({ status: "success", data: responseData, hospital });
//     }
//   } catch (error) {
//     console.error("Error saving bank:", error);
//     return res
//       .status(500)
//       .json({ status: "error", message: "Internal server error" });
//   }
// });

router.post("/save-bank", async (req, res) => {
  try {
    const {
      hospitalId,
      accountHolderName,
      country,
      selectedBankCode,
      selectedBankName,
      selectedBankBranch,
      selectedBankBranchCode,
      phoneNumber,
      accountNumber,
      bankAccountCurrency,
      isPrimary,
    } = req.body;

    const hospital = await Hospital.findById(hospitalId).select(
      "hospitalName _id email"
    );
    if (!hospital) {
      return res
        .status(404)
        .json({ status: "error", message: "Hospital not found" });
    }

    const existingAccount = await HospitalBank.findOne({ accountNumber });
    if (existingAccount) {
      return res.status(400).json({
        status: "error",
        message: "Account already exists",
      });
    }

    const dataToSend = {
      account_number: accountNumber,
      account_bank: selectedBankCode,
      bank_name: selectedBankName,
      currency: bankAccountCurrency,
      beneficiary_name: accountHolderName,
      destination_branch_code: selectedBankBranchCode,
    };

    let flutterwaveResponse;
    let status;
    let errorMessage;

    try {
      const response = await axios.post(
        "https://api.flutterwave.com/v3/beneficiaries",
        dataToSend,
        {
          headers: {
            Authorization: `Bearer ${process.env.FLW_SECRET_KEY}`,
            "Content-Type": "application/json",
          },
        }
      );

      flutterwaveResponse = response.data;
    } catch (error) {
      if (error.response && error.response.data) {
        flutterwaveResponse = error.response.data;
      }
    }

    if (flutterwaveResponse.status === "success") {
      status = "pending";
    } else {
      status = "declined";
      errorMessage = flutterwaveResponse.message;
    }

    const responseData = {
      hospitalId,
      accountHoldersName: accountHolderName,
      bankCountry: country,
      bankCode: selectedBankCode,
      bankName: selectedBankName,
      bankBranch: selectedBankBranch,
      bankBranchCode: selectedBankBranchCode,
      accountNumber,
      phoneNumber,
      bankAccountCurrency,
      status,
      errorMessage: errorMessage || null,
      isPrimary,
      flutterwaveResponseData: flutterwaveResponse,
    };

    const hospitalBank = new HospitalBank(responseData);
    await hospitalBank.save();

    return res.status(200).json({
      status: "success",
      message: "Bank account saved successfully",
      data: responseData,
    });
  } catch (error) {
    console.error("Error saving bank:", error);

    return res.status(500).json({
      status: "error",
      message: "Internal server error",
    });
  }
});

// Define a route to get all transactions requested from hospitals
router.get("/admin/transactions", async (req, res) => {
  try {
    const transactions = await Transaction.find()
      .populate("hospitalId", "_id hospitalName")
      .populate(
        "hospitalBankId",
        "_id bankName bankCode bankBranch accountNumber bankCountry flutterwaveResponseData.data bankAccountCurrency"
      );
    res.status(200).json({ status: "success", data: transactions });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Define a route to get a specific transaction by ID
router.get("/admin/transaction/:id", async (req, res) => {
  const transactionId = req.params.id;

  try {
    // Find the transaction by ID and populate related data
    const transaction = await Transaction.findById(transactionId)
      .populate("hospitalId", "_id hospitalName")
      .populate(
        "hospitalBankId",
        "_id bankName bankCode bankBranch accountNumber bankCountry flutterwaveResponseData.data bankAccountCurrency"
      );

    if (!transaction) {
      return res
        .status(404)
        .json({ status: "error", message: "Transaction not found" });
    }

    res.status(200).json({
      status: "success",
      transaction: transaction,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ status: "error", message: "Internal Server Error" });
  }
});

// Define a route to approve or decline a transaction by ID
router.post("/admin/transaction/:id", async (req, res) => {
  const transactionId = req.params.id;
  const { status } = req.body;

  try {
    const transaction = await Transaction.findById(transactionId).populate(
      "hospitalBankId"
    );

    if (!transaction) {
      return res
        .status(404)
        .json({ status: "error", message: "Transaction not found" });
    }

    if (transaction.status !== "pending") {
      return res
        .status(400)
        .json({ status: "error", message: "Transaction is not pending" });
    }

    if (status === "approved") {
      transaction.status = "approved";
      await transaction.save();

      const hospitalBankData =
        transaction.hospitalBankId.flutterwaveResponseData.data;

      // Create a notification for the hospital
      const notificationContent = `Your withdrawal request has been approved and in processing. You will received withdrawal in 2 bussiness days. Amount: ${transaction.amount} ${transaction.hospitalBankId.bankAccountCurrency}, Reference: ${transaction.transId}`;
      createHospitalNotification(
        transaction.hospitalId,
        "withdrawal",
        notificationContent,
        transaction._id
      );

      const requestData = {
        beneficiary: hospitalBankData.id,
        beneficiary_name: hospitalBankData.full_name,
        account_bank: hospitalBankData.bank_code,
        account_number: hospitalBankData.account_number,
        amount: transaction.amount,
        reference: transaction.transId,
        currency: transaction.hospitalBankId.bankAccountCurrency,
        destination_branch_code: transaction.hospitalBankId.bankBranchCode,
        narration: "LetoSave Withdrawal",
      };

      try {
        const response = await axios.post(
          "https://api.flutterwave.com/v3/transfers",
          requestData,
          {
            headers: {
              Authorization: `Bearer ${process.env.FLW_SECRET_KEY}`,
            },
          }
        );
        transaction.transferResponse = response.data;
        await transaction.save();
      } catch (error) {
        transaction.transferResponse = error.response.data;
        await transaction.save();
      }

      res.status(200).json({
        status: "success",
        message: "Transaction approved and processing",
        data: requestData,
      });
    } else if (status === "declined") {
      transaction.status = "declined";
      await transaction.save();

      const hospital = await Hospital.findById(transaction.hospitalId);
      if (hospital) {
        hospital.balance += transaction.amount;
        await hospital.save();
      }

      // Create a notification for the hospital
      const notificationContent = `Your withdrawal request has been declined. Amount: ${transaction.amount} ${transaction.hospitalBankId.bankAccountCurrency}, Reference: ${transaction.transId}`;
      createHospitalNotification(
        transaction.hospitalId,
        "withdrawal",
        notificationContent,
        transaction._id
      );

      res.status(200).json({
        status: "success",
        message: "Transaction declined, amount refunded to hospital balance",
        data: {
          beneficiary:
            transaction.hospitalBankId.flutterwaveResponseData.data.id,
          amount: transaction.amount,
          reference: transaction.transId,
          currency: transaction.hospitalBankId.bankAccountCurrency,
        },
      });
    } else {
      return res
        .status(400)
        .json({ status: "error", message: "Invalid status provided" });
    }
  } catch (error) {
    console.error(error);
    res.status(500).json({ status: "error", message: "Internal Server Error" });
  }
});

module.exports = router;
