const express = require("express");
const router = express.Router();
const jwt = require("jsonwebtoken");
const { v4: uuidv4 } = require("uuid");
const sendEmail = require("../../utils/sendEmail");
const verifyToken = require("../../middlewares/verifyToken");
const { hashPassword, comparePasswords } = require("../../helpers/hashPass");
const jwtSecret = process.env.JWT_SECRET_KEY;
const {
  Patient,
  PatientService,
  PatientTransaction,
  PatientNotification,
} = require("../../models/patient");
const {
  Hospital,
  Service,
  HospitalNotification,
  HospitalBank,
  CommentReview,
} = require("../../models/hospital");
const { upload } = require("../../config/multerConfig");
const africasTalking = require("../../config/africasTalkingConfig");

async function sendVerificationCode(phoneNumber, verificationCode) {
  africasTalking.SMS.send({
    to: `${phoneNumber}`,
    message: `Your LetoSave verification code is ${verificationCode}`,
  })
    .then((res) => console.log("Message send successfully", res))
    .catch((err) => console.log(err));
}

// Function to create a hospital notification
async function createPatientNotification(patientId, title, content) {
  try {
    // Check if a PatientNotification document already exists for the patientId
    const existingNotification = await PatientNotification.findOne({
      patientId,
    });

    if (existingNotification) {
      existingNotification.notifications.push({
        title,
        content,
      });

      await existingNotification.save();
    } else {
      const notification = new PatientNotification({
        patientId,
        notifications: [
          {
            title,
            content,
          },
        ],
      });

      await notification.save();
    }
  } catch (error) {
    console.error("Error creating hospital notification:", error);
  }
}

// Function to create a hospital notification
async function createHospitalNotification(
  hospitalId,
  title,
  content,
  transactionId
) {
  try {
    // Check if a HospitalNotification document already exists for the hospitalId
    const existingNotification = await HospitalNotification.findOne({
      hospitalId,
    });

    if (existingNotification) {
      existingNotification.notifications.push({
        title,
        content,
        transactionId,
      });

      await existingNotification.save();
    } else {
      const notification = new HospitalNotification({
        hospitalId,
        notifications: [
          {
            title,
            content,
            transactionId,
          },
        ],
      });

      await notification.save();
    }
  } catch (error) {
    console.error("Error creating hospital notification:", error);
  }
}

// Route Patient Register
router.post("/patient-register", async (req, res) => {
  try {
    const {
      name,
      email,
      phone,
      //  password
    } = req.body;

    const existingPatient = await Patient.findOne({
      $or: [{ email }, { phone }],
    });

    if (existingPatient) {
      if (!existingPatient.isPhoneVerified) {
        const verificationCode = Math.floor(
          1000 + Math.random() * 9000
        ).toString();
        const verificationCodeExpire = new Date(Date.now() + 3600 * 1000);

        existingPatient.name = name;
        existingPatient.email = email;
        existingPatient.phone = phone;
        // existingPatient.password = await hashPassword(password);
        existingPatient.phoneOtp = verificationCode;
        existingPatient.phoneOtpExpire = verificationCodeExpire;

        sendVerificationCode(phone, verificationCode);

        await existingPatient.save();

        const payload = { patientId: existingPatient._id };
        const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

        return res
          .status(200)
          .json({ message: "Verification code resent successfully", token });
      }

      const payload = { patientId: existingPatient._id };
      const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

      return res.status(200).json({
        message: "Patient with the provided email or phone already exists",
        token,
      });
    }

    const verificationCode = Math.floor(1000 + Math.random() * 9000).toString();
    const verificationCodeExpire = new Date(Date.now() + 3600 * 1000);

    // const hashedPassword = await hashPassword(password);

    const newPatient = new Patient({
      name,
      email,
      phone,
      // password: hashedPassword,
      phoneOtp: verificationCode,
      phoneOtpExpire: verificationCodeExpire,
    });

    sendVerificationCode(phone, verificationCode);

    await newPatient.save();

    const payload = { patientId: newPatient._id };
    const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

    res
      .status(201)
      .json({ message: "Verification code sent successfully", token });
  } catch (error) {
    console.error("Error registering/updating patient:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Route to verify OTP
router.post("/patient-verify-otp", verifyToken, async (req, res) => {
  try {
    const { otp } = req.body;
    const { patientId } = req.decodedToken;

    const patient = await Patient.findById(patientId);

    if (!patient) {
      return res.status(404).json({ message: "Patient not found" });
    }

    if (patient.phoneOtp === otp && patient.phoneOtpExpire > new Date()) {
      patient.isPhoneVerified = true;
      patient.phoneOtp = undefined;
      patient.phoneOtpExpire = undefined;

      await patient.save();

      return res.status(200).json({ message: "Phone verification successful" });
    } else {
      return res.status(400).json({ message: "Invalid OTP or OTP expired" });
    }
  } catch (error) {
    console.error("Error verifying OTP:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Route for patient login
router.post("/patient-login", async (req, res) => {
  try {
    const {
      phone,
      //  password
    } = req.body;

    const patient = await Patient.findOne({ phone });

    if (!patient) {
      return res.status(401).json({ message: "Invalid phone or password" });
    }

    // const isPasswordValid = await comparePasswords(password, patient.password);

    // if (!isPasswordValid) {
    //   return res.status(401).json({ message: "Invalid phone or password" });
    // }

    if (!patient.isPhoneVerified) {
      return res.status(401).json({ message: "Phone is not verified" });
    }

    const payload = { patientId: patient._id };
    const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

    res.status(200).json({ message: "Login successful", token });
  } catch (error) {
    console.error("Error logging in patient:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Send Reset Link
router.post("/patient-forgot-password", async (req, res) => {
  try {
    const { email } = req.body;

    const patient = await Patient.findOne({ email });
    if (!patient) {
      return res.status(404).json({ message: "Patient not found" });
    }

    const forgotToken = uuidv4();
    const oneHourLater = new Date();
    oneHourLater.setHours(oneHourLater.getHours() + 1);

    patient.forgotToken = forgotToken;
    patient.forgotTokenExpire = oneHourLater;
    await patient.save();

    const resetLink = `${process.env.CLIENT_URI}/create-password/${forgotToken}`;
    const emailSubject = "Patient Password Reset";
    const emailContent = `To reset your password, click the link below:\n${resetLink}`;
    sendEmail(patient.email, emailSubject, emailContent);

    res
      .status(200)
      .json({ message: "Password reset email sent successfully", resetLink });
  } catch (error) {
    console.error("Error sending password reset email:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Check token
router.get("/patient-set-password/:id", async (req, res) => {
  try {
    const forgotToken = req.params.id;
    const patient = await Patient.findOne({ forgotToken });

    if (!patient) {
      return res.status(404).json({ message: "Patient not found" });
    }

    if (!patient.forgotToken || !patient.forgotTokenExpire) {
      return res.status(400).json({ message: "Token not found" });
    }

    const tokenExpiration = new Date(patient.forgotTokenExpire);
    if (tokenExpiration <= new Date()) {
      return res.status(400).json({ message: "Token has expired" });
    }

    res.status(200).json({ message: "Token is valid" });
  } catch (error) {
    console.error("Error checking token validity:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Reset Password first check token
router.post("/patient-set-password/:id", async (req, res) => {
  try {
    const forgotToken = req.params.id;
    const password = req.body.password;

    const patient = await Patient.findOne({ forgotToken });

    if (!patient) {
      return res.status(404).json({ message: "Patient not found" });
    }

    if (!patient.forgotToken || !patient.forgotTokenExpire) {
      return res.status(400).json({ message: "Token not found" });
    }

    const tokenExpiration = new Date(patient.forgotTokenExpire);
    if (tokenExpiration <= new Date()) {
      return res.status(400).json({ message: "Token has expired" });
    }

    const hashedPassword = await hashPassword(password);

    patient.password = hashedPassword;
    patient.forgotToken = undefined;
    patient.forgotTokenExpire = undefined;

    await patient.save();

    const payload = { patientId: patient._id };
    const token = jwt.sign(payload, jwtSecret, { expiresIn: "10h" });

    res.status(200).json({ message: "Password reset successful", token });
  } catch (error) {
    console.error("Error resetting password:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Patient Details
router.get("/patient-details", verifyToken, async (req, res) => {
  try {
    const { patientId } = req.decodedToken;

    const patientDetails = await Patient.findById(patientId);

    if (!patientDetails) {
      return res.status(404).json({ error: "Patient not found" });
    }

    res.json({ patientDetails });
  } catch (error) {
    console.error("Error fetching patient details:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Change Password
router.post("/patient-change-password", verifyToken, async (req, res) => {
  try {
    const { currentPassword, newPassword } = req.body;
    const { patientId } = req.decodedToken;

    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res.status(404).json({ message: "Patient not found" });
    }

    const isPasswordValid = await comparePasswords(
      currentPassword,
      patient.password
    );
    if (!isPasswordValid) {
      return res.status(400).json({ message: "Current password is incorrect" });
    }

    const hashedNewPassword = await hashPassword(newPassword);
    patient.password = hashedNewPassword;
    await patient.save();

    res.status(200).json({ message: "Password updated successfully" });
  } catch (error) {
    console.error("Error updating password:", error);
    res.status(500).json({ message: "Internal server error" });
  }
});

// Route to update patient profile including profile picture
router.put(
  "/patient-update-profile",
  verifyToken,
  upload.single("profilePicture"),
  async (req, res) => {
    try {
      const { name, email, dateOfBirth, expectedDelivery, childsGender } =
        req.body;
      const { patientId } = req.decodedToken;

      const patient = await Patient.findById(patientId);

      if (!patient) {
        return res.status(404).json({ error: "Patient not found" });
      }

      patient.name = name;
      patient.email = email;
      patient.dateOfBirth = dateOfBirth;
      patient.expectedDelivery = expectedDelivery;
      patient.childsGender = childsGender;

      if (req.file) {
        const fileName = req.file.filename;
        patient.profilePicture = fileName;
      }

      await patient.save();

      res.status(200).json({ message: "Profile updated successfully" });
    } catch (error) {
      console.error("Error updating patient profile:", error);
      res.status(500).json({ error: "Internal server error" });
    }
  }
);

// Route to get all hospitals' details
router.get("/hospitals", async (req, res) => {
  try {
    const hospitals = await Hospital.find({}, { password: 0 });

    res.status(200).json({ hospitals });
  } catch (error) {
    console.error("Error fetching hospitals:", error);
    res.status(500).json({ error: "Internal server error" });
  }
});

// Service Management
// Route for patients to request a service with notifications
router.post("/patient-add-service", async (req, res) => {
  try {
    const { serviceId, patientId } = req.body;

    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res.status(404).json({ error: "Patient not found" });
    }
    const service = await Service.findById(serviceId);
    if (!service) {
      return res.status(404).json({ error: "service not found" });
    }

    const newPatientService = new PatientService({
      service: serviceId,
      patient: patientId,
      hospitalId: service.hospitalId,
      reBalance: service.serviceCost,
      time: new Date(),
      status: "pending",
    });

    await newPatientService.save();

    patient.services.push(newPatientService._id);
    await patient.save();

    const notificationTitle = "Service Request";
    const notificationContent = `Service request from Patient: ${patient.name} for Service: ${service.serviceName} has been delivered to the hospital. Please wait for service response.`;
    const transactionId = serviceId;

    await createHospitalNotification(
      service.hospitalId,
      notificationTitle,
      notificationContent,
      transactionId
    );

    res.status(200).json({ message: "Service added successfully" });
  } catch (error) {
    console.error("Error adding service:", error);
    res
      .status(500)
      .json({ error: "An error occurred while adding the service" });
  }
});

// Route for patients to create a transaction with notifications
router.post("/patient-create-transaction", async (req, res) => {
  try {
    const {
      patientId,
      patientServiceId,
      hospitalBankId,
      amount: amountValue,
      transactionId,
      status,
      referenceId,
      flutterwaveResponse,
      paymentBy,
    } = req.body;

    const amount =
      typeof amountValue === "string" ? parseFloat(amountValue) : amountValue;

    if (isNaN(amount)) {
      return res.status(400).json({ error: "Invalid amount format" });
    }

    const patient = await Patient.findOne({ _id: patientId });
    if (!patient) {
      return res.status(404).json({ error: "Patient not found" });
    }
    const hospitalBank = await HospitalBank.findOne({ _id: hospitalBankId });
    if (!hospitalBank) {
      return res.status(404).json({ error: "Hospital Bank not found" });
    }
    const hospital = await Hospital.findOne({ _id: hospitalBank.hospitalId });
    if (!hospital) {
      return res.status(404).json({ error: "Hospital not found" });
    }
    const patientService = await PatientService.findOne({
      _id: patientServiceId,
    }).populate("service");
    if (!patientService) {
      return res.status(404).json({ error: "Patient Service not found" });
    }
    if (patientService.status !== "approved") {
      return res.status(400).json({
        error: "Service status is not approved. Cannot transfer amount.",
      });
    }
    if (patientService.reBalance < amount) {
      return res.status(400).json({
        error: `Your reBalance is ${patientService.reBalance}. You can transfer only an amount equal to or less than your reBalance.`,
      });
    }

    let patientNotificationTitle = "";
    let patientNotificationContent = "";

    if (status === "successful") {
      const transaction = new PatientTransaction({
        patientId,
        hospitalId: patientService.hospitalId,
        hospitalBankId,
        patientServiceId,
        serviceId: patientService.service,
        amount,
        referenceId,
        transactionId,
        reBalance: patientService.reBalance - amount,
        status: "approved",
        flutterwaveResponse,
        paymentBy,
      });
      await transaction.save();

      patientService.amountSaved += amount;
      patientService.reBalance -= amount;
      hospital.balance += amount;
      await Promise.all([patientService.save(), hospital.save()]);

      patientNotificationTitle = "Payment Successful";
      patientNotificationContent = `Your payment of ${amount} for service: ${patientService.service.serviceName} was successful.`;
    } else {
      const transaction = new PatientTransaction({
        patientId,
        hospitalId: patientService.hospitalId,
        hospitalBankId,
        patientServiceId,
        serviceId: patientService.service,
        amount,
        referenceId,
        transactionId,
        reBalance: patientService.reBalance,
        status: "declined",
        flutterwaveResponse,
        paymentBy,
      });
      await transaction.save();

      patientNotificationTitle = "Payment Declined";
      patientNotificationContent = `Your payment of ${amount} for service: ${patientService.service.serviceName} was declined.`;
    }

    // Create a patient notification
    await createPatientNotification(
      patientId,
      patientNotificationTitle,
      patientNotificationContent
    );

    res.status(201).json({ message: "Transaction processed successfully" });
  } catch (error) {
    console.error(error);
    res.status(500).json({ error: "Internal Server Error" });
  }
});

// Route to show all transactions for a specific patient by their ID
router.get("/patient-transactions/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;
    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res
        .status(404)
        .json({ status: false, error: "Patient not found" });
    }

    const patientTransactions = await PatientTransaction.find({ patientId });

    const transactionDetails = [];

    const serviceReBalances = {};
    const serviceAmountsPaid = {};

    for (const transaction of patientTransactions) {
      const patientService = await PatientService.findById(
        transaction.patientServiceId
      );
      if (patientService) {
        const patientServiceId = patientService._id.toString();

        if (!serviceReBalances[patientServiceId]) {
          serviceReBalances[patientServiceId] = 0;
          serviceAmountsPaid[patientServiceId] = 0;
        }

        serviceReBalances[patientServiceId] = transaction.reBalance;
        serviceAmountsPaid[patientServiceId] += transaction.amount;

        const service = await Service.findById(patientService.service);
        if (service) {
          transactionDetails.push({
            transactionId: transaction._id,
            patientServiceId: patientServiceId,
            hospitalId: transaction.hospitalId,
            serviceName: service.serviceName,
            reBalance: transaction.reBalance,
            amount: transaction.amount,
            status: transaction.status,
            time: transaction.time,
          });
        }
      }
    }

    // Calculate the total amount paid and total rebalance
    let totalAmountPaid = 0;
    let totalRebalance = 0;

    for (const patientServiceId in serviceAmountsPaid) {
      totalAmountPaid += serviceAmountsPaid[patientServiceId];
      totalRebalance += serviceReBalances[patientServiceId];
    }

    res.status(200).json({
      status: true,
      totalAmountPaid,
      totalRebalance,
      serviceReBalances,
      serviceAmountsPaid,
      patientTransactions: transactionDetails,
    });
  } catch (error) {
    console.error("Error fetching patient transactions:", error);
    res.status(500).json({
      status: false,
      error: "An error occurred while fetching patient transactions",
    });
  }
});

// Route to show all transactions for a specific patient by their ID
router.get("/trimester/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;
    const { year, month } = req.body;

    if (!year || (!month && month !== 0) || month < 1 || month > 10) {
      return res.status(400).json({
        status: false,
        error: "Invalid year or month. Month must be between 1 and 10.",
      });
    }

    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res
        .status(404)
        .json({ status: false, error: "Patient not found." });
    }

    const patientTransactions = await PatientTransaction.find({ patientId });

    const transactionDetails = [];

    for (const transaction of patientTransactions) {
      const patientService = await PatientService.findById(
        transaction.patientServiceId
      );
      if (patientService) {
        const patientServiceId = patientService._id.toString();

        const service = await Service.findById(patientService.service);
        if (service) {
          transactionDetails.push({
            transactionId: transaction._id,
            patientServiceId: patientServiceId,
            hospitalId: transaction.hospitalId,
            serviceName: service.serviceName,
            reBalance: transaction.reBalance,
            amount: transaction.amount,
            status: transaction.status,
            time: transaction.time,
          });
        }
      }
    }

    // Use the filterAndSummarizeTransactions function here
    function filterAndSummarizeTransactions(
      transactionsData,
      filterYear,
      filterMonth
    ) {
      const filteredTransactions = transactionsData.filter((transaction) => {
        const transactionDate = new Date(transaction.time);
        const transactionYear = transactionDate.getFullYear();
        const transactionMonth = transactionDate.getMonth();

        return (
          (transactionYear === filterYear && transactionMonth >= filterMonth) ||
          (transactionYear === filterYear + 1 && transactionMonth < filterMonth)
        );
      });

      const monthSummaries = {};
      for (let i = 0; i < 3; i++) {
        const currentMonth = (filterMonth - 1 + i) % 12;
        const currentYear = filterYear + Math.floor((filterMonth - 1 + i) / 12);
        const key = `${currentYear}-${(currentMonth + 1)
          .toString()
          .padStart(2, "0")}`;
        const transactionsInMonth = filteredTransactions.filter(
          (transaction) => {
            const transactionDate = new Date(transaction.time);
            const transactionYear = transactionDate.getFullYear();
            const transactionMonth = transactionDate.getMonth();

            return (
              transactionYear === currentYear &&
              transactionMonth === currentMonth &&
              transaction.status === "approved"
            );
          }
        );
        monthSummaries[key] = transactionsInMonth.reduce(
          (total, transaction) => total + transaction.amount,
          0
        );
      }

      return {
        monthSummaries,
      };
    }

    if (transactionDetails.length > 0) {
      const result = filterAndSummarizeTransactions(
        transactionDetails,
        year,
        month
      );
      res.status(200).json({
        status: true,
        data: result,
      });
    } else {
      // Handle case when there are no transaction details.
      res.status(200).json({
        status: true,
        data: {
          monthSummaries: {},
        },
      });
    }
  } catch (error) {
    console.error("Error fetching patient transactions:", error);
    res.status(500).json({
      status: false,
      error: "An error occurred while fetching patient transactions",
    });
  }
});

// Route to show all transactions for a specific patient by their ID
router.get("/weekly/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;
    const { year, week } = req.body;

    if (!year || (!week && week !== 0) || week < 1 || week > 52) {
      return res.status(400).json({
        status: false,
        error: "Invalid year or week. week must be between 1 and 52.",
      });
    }

    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res
        .status(404)
        .json({ status: false, error: "Patient not found." });
    }

    const patientTransactions = await PatientTransaction.find({ patientId });

    const transactionDetails = [];

    for (const transaction of patientTransactions) {
      const patientService = await PatientService.findById(
        transaction.patientServiceId
      );
      if (patientService) {
        const patientServiceId = patientService._id.toString();

        const service = await Service.findById(patientService.service);
        if (service) {
          transactionDetails.push({
            transactionId: transaction._id,
            patientServiceId: patientServiceId,
            hospitalId: transaction.hospitalId,
            serviceName: service.serviceName,
            reBalance: transaction.reBalance,
            amount: transaction.amount,
            status: transaction.status,
            time: transaction.time,
          });
        }
      }
    }

    function getWeekTotalApprovedAmount(week, year, transactionData) {
      const startDate = new Date(year, 0, 1);
      const endDate = new Date(year, 11, 31);
      const weekStart = new Date(startDate.getTime());
      const weekEnd = new Date(startDate.getTime());

      // Set the weekStart and weekEnd to the first day of the desired week
      weekStart.setDate(startDate.getDate() + (week - 1) * 7);
      weekEnd.setDate(startDate.getDate() + (week - 1) * 7 + 6);

      const dayNames = [
        "Sunday",
        "Monday",
        "Tuesday",
        "Wednesday",
        "Thursday",
        "Friday",
        "Saturday",
      ];
      const dayWiseApprovedAmounts = {
        Sunday: 0,
        Monday: 0,
        Tuesday: 0,
        Wednesday: 0,
        Thursday: 0,
        Friday: 0,
        Saturday: 0,
      };

      let totalApprovedAmount = 0;

      // Iterate through patient transactions and calculate day-wise and total approved amounts within the specified week
      transactionData.forEach((transaction) => {
        // Use transactionData as an argument
        const transactionDate = new Date(transaction.time);
        if (
          transactionDate >= weekStart &&
          transactionDate <= weekEnd &&
          transaction.status === "approved"
        ) {
          const dayOfWeek = transactionDate.getUTCDay(); // Get the day of the week (0-6, where 0 is Sunday)
          const dayName = dayNames[dayOfWeek];
          dayWiseApprovedAmounts[dayName] += transaction.amount;
          totalApprovedAmount += transaction.amount;
        }
      });

      return { dayWiseApprovedAmounts, totalApprovedAmount };
    }

    if (transactionDetails.length > 0) {
      const result = getWeekTotalApprovedAmount(week, year, transactionDetails);
      res.status(200).json({
        status: true,
        data: result,
      });
    } else {
      // Handle case when there are no transaction details.
      res.status(200).json({
        status: true,
        data: {
          monthSummaries: {},
        },
      });
    }
  } catch (error) {
    console.error("Error fetching patient transactions:", error);
    res.status(500).json({
      status: false,
      error: "An error occurred while fetching patient transactions",
    });
  }
});

// Route to show a summary of transactions for a specific patient by year
router.get("/yearly/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;
    const { year } = req.body;

    if (!year || typeof year !== "number") {
      return res
        .status(400)
        .json({ status: false, error: "Invalid year provided." });
    }

    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res
        .status(404)
        .json({ status: false, error: "Patient not found." });
    }

    const patientTransactions = await PatientTransaction.find({ patientId });

    const monthNames = [
      "January",
      "February",
      "March",
      "April",
      "May",
      "June",
      "July",
      "August",
      "September",
      "October",
      "November",
      "December",
    ];

    const monthSummaries = {};

    // Initialize month summaries with zero values
    monthNames.forEach((monthName) => {
      monthSummaries[monthName] = 0;
    });

    // Iterate through patient transactions and calculate monthly summaries for the specified year
    patientTransactions.forEach((transaction) => {
      const transactionDate = new Date(transaction.time);
      if (
        transactionDate.getFullYear() === year &&
        transaction.status === "approved"
      ) {
        const monthName = monthNames[transactionDate.getMonth()];
        monthSummaries[monthName] += transaction.amount;
      }
    });

    res.status(200).json({
      status: true,
      data: {
        year,
        monthSummaries,
      },
    });
  } catch (error) {
    console.error("Error fetching patient transactions:", error);
    res.status(500).json({
      status: false,
      error: "An error occurred while fetching patient transactions",
    });
  }
});

// Notification Management Patient
// Route to create a custom notification for a patient
router.post("/create-patient-notification", async (req, res) => {
  try {
    const { patientId, title, content } = req.body;

    // Check if the patient exists
    const patient = await Patient.findById(patientId);
    if (!patient) {
      return res
        .status(404)
        .json({ status: false, error: "Patient not found" });
    }

    // Create a patient notification
    await createPatientNotification(patientId, title, content);

    res.status(201).json({
      status: true,
      message: "Custom notification created successfully",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ status: false, error: "Internal Server Error" });
  }
});

// Route to get all notifications by patientId with populated details
router.get("/patient-notifications/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;
    const patientNotifications = await PatientNotification.findOne({
      patientId: patientId,
    });
    if (!patientNotifications) {
      return res
        .status(404)
        .json({ status: false, message: "Patient notifications not found" });
    }
    const notifications = patientNotifications.notifications;
    res.status(200).json({ status: true, notifications });
  } catch (error) {
    console.error("Error fetching Patient notifications:", error);
    res.status(500).json({ status: false, error: "Internal server error" });
  }
});

// Route to update isRead status by patientId for notifications with isRead set to false
router.put("/patient-update-is-read/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;

    const result = await PatientNotification.updateMany(
      { patientId: patientId, "notifications.isRead": false },
      {
        $set: {
          "notifications.$[elem].isRead": true,
          "notifications.$[elem].isReadTime": new Date(),
        },
      },
      { arrayFilters: [{ "elem.isRead": false }] }
    );

    if (result.nModified === 0) {
      return res.status(404).json({
        status: false,
        message: "No unread notifications found to update",
      });
    }

    res
      .status(200)
      .json({ status: true, message: "isRead status updated successfully" });
  } catch (error) {
    console.error("Error updating isRead status:", error);
    res.status(500).json({ status: false, error: "Internal server error" });
  }
});

// Route to update isRemove status by patientId based on isRead and isRemove status
router.put("/patient-update-is-remove/:patientId", async (req, res) => {
  try {
    const { patientId } = req.params;
    const resultIsRead = await PatientNotification.updateMany(
      {
        patientId: patientId,
        "notifications.isRead": false,
      },
      {
        $set: {
          "notifications.$[elem].isRead": true,
          "notifications.$[elem].isReadTime": new Date(),
        },
      },
      {
        arrayFilters: [{ "elem.isRead": false }],
      }
    );
    const resultIsRemove = await PatientNotification.updateMany(
      {
        patientId: patientId,
        "notifications.isRemove": false,
      },
      {
        $set: {
          "notifications.$[elem].isRemove": true,
          "notifications.$[elem].isRemoveTime": new Date(),
        },
      },
      {
        arrayFilters: [{ "elem.isRemove": false }],
      }
    );
    if (
      (resultIsRead.nModified === 0 && resultIsRemove.nModified === 0) ||
      (resultIsRead.ok !== 1 && resultIsRemove.ok !== 1)
    ) {
      return res
        .status(404)
        .json({ status: false, message: "Update notifications success" });
    }
    res
      .status(200)
      .json({ status: true, message: "isRemove status updated successfully" });
  } catch (error) {
    console.error("Error updating isRemove status:", error);
    res.status(500).json({ status: false, error: "Internal server error" });
  }
});

// Route to update a single notification's isRead status by patientId and notificationId
router.put(
  "/patient-update-is-read/:patientId/:notificationId",
  async (req, res) => {
    try {
      const { patientId, notificationId } = req.params;

      console.log("Received patientId:", patientId);
      console.log("Received notificationId:", notificationId);

      const patientNotification = await PatientNotification.findOne({
        patientId: patientId,
        "notifications._id": notificationId,
      });

      if (!patientNotification) {
        return res
          .status(404)
          .json({ status: false, message: "Notification not found" });
      }

      const notification = patientNotification.notifications.find(
        (notif) => notif._id.toString() === notificationId
      );

      if (!notification) {
        return res
          .status(404)
          .json({ status: false, message: "Notification not found" });
      }

      if (notification.isRead) {
        return res
          .status(200)
          .json({ status: false, message: "Notification already updated" });
      }

      const result = await PatientNotification.updateOne(
        {
          patientId: patientId,
          "notifications._id": notificationId,
        },
        {
          $set: {
            "notifications.$.isRead": true,
            "notifications.$.isReadTime": new Date(),
          },
        }
      );

      if (result.nModified === 0) {
        return res
          .status(200)
          .json({ status: false, message: "Notification already updated" });
      }

      res
        .status(200)
        .json({ status: true, message: "isRead status updated successfully" });
    } catch (error) {
      console.error("Error updating isRead status:", error);
      res.status(500).json({ status: false, error: "Internal server error" });
    }
  }
);

// Route to update a single notification's isRemove status by patientId and notificationId
router.put(
  "/patient-update-is-remove/:patientId/:notificationId",
  async (req, res) => {
    try {
      const { patientId, notificationId } = req.params;

      const hospitalNotification = await PatientNotification.findOne({
        patientId: patientId,
        "notifications._id": notificationId,
      });

      if (!hospitalNotification) {
        return res
          .status(404)
          .json({ status: false, message: "Notification not found" });
      }

      const notification = hospitalNotification.notifications.find(
        (notif) => notif._id.toString() === notificationId
      );

      if (!notification) {
        return res
          .status(404)
          .json({ status: false, message: "Notification not found" });
      }

      if (notification.isRemove) {
        return res
          .status(200)
          .json({ status: false, message: "Notification already updated" });
      }

      const updateQuery = {
        $set: {
          "notifications.$.isRemove": true,
          "notifications.$.isRemoveTime": new Date(),
        },
      };

      if (!notification.isRead) {
        updateQuery.$set["notifications.$.isRead"] = true;
        updateQuery.$set["notifications.$.isReadTime"] = new Date();
      }

      const result = await PatientNotification.updateOne(
        {
          patientId: patientId,
          "notifications._id": notificationId,
        },
        updateQuery
      );

      if (result.nModified === 0) {
        return res
          .status(200)
          .json({ status: false, message: "Notification already updated" });
      }

      res.status(200).json({
        status: true,
        message: "isRemove status updated successfully",
      });
    } catch (error) {
      console.error("Error updating isRemove status:", error);
      res.status(500).json({ status: false, error: "Internal server error" });
    }
  }
);

// for comments
// Create a new comment and review
router.post("/comments-reviews", async (req, res) => {
  try {
    const { rating, comment, patientId, hospitalId } = req.body;

    const newCommentReview = new CommentReview({
      rating,
      comment,
      patientId,
      hospitalId,
    });

    await newCommentReview.save();

    res.status(201).json({
      status: true,
      message: "Comment and review created successfully",
    });
  } catch (error) {
    console.error("Error creating comment and review:", error);
    res.status(500).json({ status: false, message: "Internal server error" });
  }
});

// Get all comments and reviews for a hospital
router.get("/comments-reviews/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    const commentsReviews = await CommentReview.find({
      hospitalId,
      deletedAt: null,
    })
      .populate("hospitalId", "_id hospitalName email")
      .populate(
        "patientId",
        "_id name email phone isPhoneVerified dateOfBirth profilePicture"
      );

    res.status(200).json({ status: true, data: commentsReviews });
  } catch (error) {
    console.error("Error fetching comments and reviews:", error);
    res.status(500).json({ status: false, message: "Internal server error" });
  }
});

// Update a comment and review by ID
router.put("/comments-reviews/:id", async (req, res) => {
  try {
    const { id } = req.params;
    const { rating, comment } = req.body;

    const updatedCommentReview = await CommentReview.findByIdAndUpdate(
      id,
      { rating, comment, updatedAt: Date.now() },
      { new: true }
    );

    if (!updatedCommentReview) {
      return res
        .status(404)
        .json({ status: false, message: "Comment and review not found" });
    }

    res.status(200).json({ status: true, data: updatedCommentReview });
  } catch (error) {
    console.error("Error updating comment and review:", error);
    res.status(500).json({ status: false, message: "Internal server error" });
  }
});

// Delete a comment and review by ID
router.delete("/comments-reviews/:id", async (req, res) => {
  try {
    const { id } = req.params;

    const deletedCommentReview = await CommentReview.findByIdAndUpdate(id, {
      deletedAt: Date.now(),
    });

    if (!deletedCommentReview) {
      return res
        .status(404)
        .json({ status: false, message: "Comment and review not found" });
    }

    res.status(204).json({
      status: true,
      message: "Comment and review deleted successfully",
    });
  } catch (error) {
    console.error("Error deleting comment and review:", error);
    res.status(500).json({ status: false, message: "Internal server error" });
  }
});

// Get approved and not deleted comments and reviews for a hospital
router.get("/approved-comments-reviews/:hospitalId", async (req, res) => {
  try {
    const { hospitalId } = req.params;

    const approvedCommentsReviews = await CommentReview.find({
      hospitalId,
      isApproved: true, 
      deletedAt: null, 
    })
      .populate('hospitalId', '_id hospitalName email')
      .populate('patientId', '_id name email phone isPhoneVerified dateOfBirth profilePicture');

    res.status(200).json({ status: true, data: approvedCommentsReviews });
  } catch (error) {
    console.error("Error fetching approved comments and reviews:", error);
    res.status(500).json({ status: false, message: "Internal server error" });
  }
});


module.exports = router;



