const express = require("express");
const { Product, Cart, Category, Subcategory } = require("../../models/shop");
const { Patient } = require("../../models/patient");
const { default: mongoose } = require("mongoose");
const router = express.Router();
const { upload } = require("../../config/multerConfig");

// for category
// Create a new category
router.post("/categories", async (req, res) => {
  try {
    const { name } = req.body;

    // Check if the category name already exists
    const existingCategory = await Category.findOne({ name });
    if (existingCategory) {
      return res
        .status(400)
        .json({ success: false, message: "Category name already exists" });
    }

    const newCategory = new Category({ name });
    const savedCategory = await newCategory.save();
    res.status(201).json({ success: true, data: savedCategory });
  } catch (error) {
    console.log(error);
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Get all categories
router.get("/categories", async (req, res) => {
  try {
    const categories = await Category.find();
    res.status(200).json({ success: true, data: categories });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Get a single category by ID
router.get("/categories/:id", async (req, res) => {
  try {
    const categoryId = req.params.id;
    const category = await Category.findById(categoryId);
    if (!category) {
      return res
        .status(404)
        .json({ success: false, message: "Category not found" });
    }
    res.status(200).json({ success: true, data: category });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Update a category by ID
router.put("/categories/:id", async (req, res) => {
  try {
    const categoryId = req.params.id;
    const { name } = req.body;
    const updatedCategory = await Category.findByIdAndUpdate(
      categoryId,
      { name },
      { new: true }
    );
    if (!updatedCategory) {
      return res
        .status(404)
        .json({ success: false, message: "Category not found" });
    }
    res.status(200).json({ success: true, data: updatedCategory });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Delete a category by ID
router.delete("/categories/:id", async (req, res) => {
  try {
    const categoryId = req.params.id;
    const deletedCategory = await Category.findByIdAndRemove(categoryId);
    if (!deletedCategory) {
      return res
        .status(404)
        .json({ success: false, message: "Category not found" });
    }
    res.status(200).json({ success: true, data: deletedCategory });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// for subCategory
// Create a new subcategory
router.post("/subcategories", async (req, res) => {
  try {
    const { name, categoryId } = req.body;

    // Check if the category exists
    const category = await Category.findById(categoryId);
    if (!category) {
      return res
        .status(404)
        .json({ success: false, message: "Category not found" });
    }

    const newSubcategory = new Subcategory({ name, category: categoryId });
    const savedSubcategory = await newSubcategory.save();
    res.status(201).json({ success: true, data: savedSubcategory });
  } catch (error) {
    if (error.code === 11000) {
      // Duplicate key error (name already exists)
      return res
        .status(400)
        .json({ success: false, message: "Subcategory name already exists" });
    }
    console.log(error);
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Get all subcategories
router.get("/subcategories", async (req, res) => {
  try {
    const subcategories = await Subcategory.find().populate("category", "name");
    res.status(200).json({ success: true, data: subcategories });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Get a single subcategory by ID
router.get("/subcategories/:id", async (req, res) => {
  try {
    const subcategoryId = req.params.id;
    const subcategory = await Subcategory.findById(subcategoryId).populate(
      "category",
      "name"
    );
    if (!subcategory) {
      return res
        .status(404)
        .json({ success: false, message: "Subcategory not found" });
    }
    res.status(200).json({ success: true, data: subcategory });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Update a subcategory by ID
router.put("/subcategories/:id", async (req, res) => {
  try {
    const subcategoryId = req.params.id;
    const { name, categoryId } = req.body;

    // Check if the category exists
    const category = await Category.findById(categoryId);
    if (!category) {
      return res
        .status(404)
        .json({ success: false, message: "Category not found" });
    }

    const updatedSubcategory = await Subcategory.findByIdAndUpdate(
      subcategoryId,
      { name, category: categoryId },
      { new: true }
    );
    if (!updatedSubcategory) {
      return res
        .status(404)
        .json({ success: false, message: "Subcategory not found" });
    }
    res.status(200).json({ success: true, data: updatedSubcategory });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// Delete a subcategory by ID
router.delete("/subcategories/:id", async (req, res) => {
  try {
    const subcategoryId = req.params.id;
    const deletedSubcategory = await Subcategory.findByIdAndRemove(
      subcategoryId
    );
    if (!deletedSubcategory) {
      return res
        .status(404)
        .json({ success: false, message: "Subcategory not found" });
    }
    res.status(200).json({ success: true, data: deletedSubcategory });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// for product

// POST /api/shop/products with image upload
router.post("/products", upload.single("product-image"), async (req, res) => {
  try {
    const { name, category, subCategory, price, quantity, description } =
      req.body;

    const validCategory = await Category.findById(category);
    if (!validCategory) {
      return res.status(400).json({
        success: false,
        message: "Invalid category ID. Please provide a valid category ID.",
      });
    }

    const validSubcategory = await Subcategory.findById(subCategory);
    if (!validSubcategory) {
      return res.status(400).json({
        success: false,
        message:
          "Invalid subcategory ID. Please provide a valid subcategory ID.",
      });
    }

    if (validSubcategory.category.toString() !== validCategory._id.toString()) {
      return res.status(400).json({
        success: false,
        message: "Category and subcategory do not match.",
      });
    }

    const image = req.file ? req.file.filename : undefined;

    const newProduct = new Product({
      name,
      category: validCategory._id,
      subCategory: validSubcategory._id,
      price,
      quantity,
      description,
      image, 
    });

    const savedProduct = await newProduct.save();

    res.status(201).json({ success: true, data: savedProduct });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// // PUT /api/shop/products/:id
// router.put("/products/:id", async (req, res) => {
//   try {
//     const productId = req.params.id;

//     // Find the product by ID
//     const product = await Product.findById(productId);

//     if (!product) {
//       return res
//         .status(404)
//         .json({ success: false, message: "Product not found" });
//     }

//     // Check if the provided category ID is valid
//     const validCategory =
//       req.body.category && (await Category.findById(req.body.category));

//     // Check if the provided subcategory ID is valid
//     const validSubcategory =
//       req.body.subCategory &&
//       (await Subcategory.findById(req.body.subCategory));

//     // Check if the user wants to change the category and subcategory
//     if (req.body.category && req.body.subCategory) {
//       // Check if both the category and subcategory exist and are related
//       if (
//         !validCategory ||
//         !validSubcategory ||
//         validSubcategory.category.toString() !== validCategory._id.toString()
//       ) {
//         return res.status(400).json({
//           success: false,
//           message: "Category and subcategory do not match.",
//         });
//       }

//       // Update category and subcategory
//       product.category = validCategory._id;
//       product.subCategory = validSubcategory._id;
//     } else {
//       // Check if the user wants to change category without defining subcategory
//       if (req.body.category && !req.body.subCategory) {
//         return res.status(400).json({
//           success: false,
//           message: "Please define both category and subcategory.",
//         });
//       }

//       // Check if the user wants to change subcategory without defining category
//       if (req.body.subCategory && !req.body.category) {
//         return res.status(400).json({
//           success: false,
//           message: "Please define both category and subcategory.",
//         });
//       }
//     }

//     // Update other product fields based on the request body
//     if (req.body.name) {
//       product.name = req.body.name;
//     }
//     if (req.body.price) {
//       product.price = req.body.price;
//     }
//     if (req.body.quantity) {
//       product.quantity = req.body.quantity;
//     }
//     if (req.body.description) {
//       product.description = req.body.description;
//     }

//     // Set updatedAt to the current date
//     product.updatedAt = new Date();

//     // Save the updated product
//     const updatedProduct = await product.save();

//     res.status(200).json({ success: true, data: updatedProduct });
//   } catch (error) {
//     res.status(500).json({ success: false, message: "Internal server error" });
//   }
// });

// PUT /api/shop/products/:id
router.put("/products/:id", upload.single("product-image"), async (req, res) => {
  try {
    const productId = req.params.id;
    const product = await Product.findById(productId);
    if (!product) {
      return res
        .status(404)
        .json({ success: false, message: "Product not found" });
    }
    const validCategory =
      req.body.category && (await Category.findById(req.body.category));
    const validSubcategory =
      req.body.subCategory &&
      (await Subcategory.findById(req.body.subCategory));
    if (req.body.category && req.body.subCategory) {
      if (
        !validCategory ||
        !validSubcategory ||
        validSubcategory.category.toString() !== validCategory._id.toString()
      ) {
        return res.status(400).json({
          success: false,
          message: "Category and subcategory do not match.",
        });
      }
      product.category = validCategory._id;
      product.subCategory = validSubcategory._id;
    } else {
      if (req.body.category && !req.body.subCategory) {
        return res.status(400).json({
          success: false,
          message: "Please define both category and subcategory.",
        });
      }
      if (req.body.subCategory && !req.body.category) {
        return res.status(400).json({
          success: false,
          message: "Please define both category and subcategory.",
        });
      }
    }
    if (req.body.name) {
      product.name = req.body.name;
    }
    if (req.body.price) {
      product.price = req.body.price;
    }
    if (req.body.quantity) {
      product.quantity = req.body.quantity;
    }
    if (req.body.description) {
      product.description = req.body.description;
    }
    if (req.file) {
      if (product.image) {
        fs.unlinkSync(uploadDestination + product.image);
      }
      product.image = req.file.filename;
    }
    product.updatedAt = new Date();
    const updatedProduct = await product.save();
    res.status(200).json({ success: true, data: updatedProduct });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});


// DELETE /api/shop/products/:id
router.delete("/products/:id", async (req, res) => {
  try {
    const productId = req.params.id;

    // Find the product by ID
    const product = await Product.findById(productId);

    if (!product) {
      return res
        .status(404)
        .json({ success: false, message: "Product not found" });
    }

    // Soft delete the product by setting the deletedAt field
    product.deletedAt = new Date();

    // Save the updated product to mark it as deleted
    const deletedProduct = await product.save();

    res.status(200).json({ success: true, data: deletedProduct });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});


// GET /api/shop/products
router.get("/products", async (req, res) => {
  try {
    const products = await Product.find({ deletedAt: null })
      .populate("category", "name")
      .populate("subCategory", "name");

    res.status(200).json({ success: true, data: products });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});


// GET /api/shop/products/:id
router.get("/products/:id", async (req, res) => {
  try {
    const productId = req.params.id;

    // Find the product by ID and populate the category and subCategory fields
    const product = await Product.findById(productId)
      .populate("category", "name")
      .populate("subCategory", "name");

    if (!product) {
      return res
        .status(404)
        .json({ success: false, message: "Product not found" });
    }

    res.status(200).json({ success: true, data: product });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// GET categories and related sub-categories
router.get("/categories-with-subcategories", async (req, res) => {
  try {
    // Find all unique categories
    const uniqueCategories = await Category.find({});

    // Create an array to store the formatted data
    const categoryData = [];

    // Iterate through the unique categories and find related sub-categories
    for (const category of uniqueCategories) {
      const subCategories = await Subcategory.find({ category: category._id });
      categoryData.push({
        category: {
          _id: category._id,
          name: category.name,
        },
        subCategories: subCategories.map((subCategory) => ({
          _id: subCategory._id,
          name: subCategory.name,
        })),
      });
    }

    res.status(200).json({
      success: true,
      data: categoryData,
    });
  } catch (error) {
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// GET related products by category and sub-category
router.get("/related-products", async (req, res) => {
  try {
    const { category, subCategory } = req.query;
    const query = {};

    if (category) {
      query.category = category;
    }

    if (subCategory) {
      query.subCategory = subCategory;
    }

    // Exclude products with a "deletedAt" date
    query.deletedAt = null;

    // Find related products based on the category, sub-category, and not deleted
    const relatedProducts = await Product.find(query)
      .populate("category", "name")
      .populate("subCategory", "name");

    if (relatedProducts.length === 0) {
      return res.status(404).json({
        success: false,
        message:
          "No related products found for the specified category and sub-category.",
      });
    }

    res.status(200).json({
      success: true,
      data: relatedProducts,
    });
  } catch (error) {
    console.log(error);
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// for cart
// POST /api/shop/cart
router.post("/cart", async (req, res) => {
  try {
    const { PatientId, productId, quantity, action } = req.body;

    // Check if PatientId is a valid ObjectId
    if (!mongoose.Types.ObjectId.isValid(PatientId)) {
      return res.status(400).json({
        success: false,
        message: "Invalid PatientId. Please provide a valid Patient Id.",
      });
    }

    // Find the patient by PatientId
    const patient = await Patient.findById(PatientId);

    if (!patient) {
      return res
        .status(404)
        .json({ success: false, message: "Patient not found" });
    }

    // Find the patient's cart or create a new one if it doesn't exist
    let cart = await Cart.findOne({ patientId: patient._id });

    if (!cart) {
      cart = new Cart({ patientId: patient._id, items: [] });
    }

    // Find the product by ID
    const product = await Product.findById(productId);

    if (!product) {
      return res
        .status(404)
        .json({ success: false, message: "Product not found" });
    }

    // Find the item in the cart based on the productId
    const cartItem = cart.items.find(
      (item) => item.productId.toString() === productId
    );

    if (action === "add") {
      // If the item exists in the cart, increase the quantity
      if (cartItem) {
        cartItem.quantity += quantity;
      } else {
        // If the item doesn't exist, add it to the cart
        cart.items.push({
          productId,
          quantity,
        });
      }
    } else if (action === "remove") {
      // If the item exists in the cart and the requested quantity is less than or equal to the current quantity, decrease the quantity
      if (cartItem && cartItem.quantity >= quantity) {
        cartItem.quantity -= quantity;
        // Remove the item from the cart if the quantity becomes zero
        if (cartItem.quantity === 0) {
          cart.items = cart.items.filter(
            (item) => item.productId.toString() !== productId
          );
        }
      } else {
        return res.status(400).json({
          success: false,
          message: "Invalid quantity or item not found in the cart",
        });
      }
    } else {
      return res.status(400).json({
        success: false,
        message: "Invalid action. Use 'add' or 'remove'.",
      });
    }

    // Save the updated cart
    await cart.save();

    // Respond with the updated cart data
    res.status(200).json({
      success: true,
      message: "Cart updated successfully",
      data: cart,
    });
  } catch (error) {
    console.log(error);
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// GET /api/shop/cart/:PatientId
router.get("/cart/:PatientId", async (req, res) => {
  try {
    const { PatientId } = req.params;

    // Check if PatientId is a valid ObjectId
    if (!mongoose.Types.ObjectId.isValid(PatientId)) {
      return res.status(400).json({
        success: false,
        message: "Invalid PatientId. Please provide a valid Patient Id.",
      });
    }

    // Find the patient by PatientId
    const patient = await Patient.findById(PatientId);

    if (!patient) {
      return res
        .status(404)
        .json({ success: false, message: "Patient not found" });
    }

    // Find the patient's cart and populate items with their products and categories
    const cart = await Cart.findOne({ patientId: patient._id }).populate({
      path: "items",
      populate: {
        path: "productId",
        populate: {
          path: "category subCategory",
          select: "name",
        },
        select: "name price",
      },
    });

    if (!cart) {
      return res
        .status(404)
        .json({ success: false, message: "Cart not found" });
    }

    res.status(200).json({ success: true, data: { patient, cart } });
  } catch (error) {
    console.error(error);
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

// POST /api/shop/checkout/:cartId
router.post("/checkout/:cartId", async (req, res) => {
  try {
    const { cartId } = req.params;

    // Check if cartId is a valid ObjectId
    if (!mongoose.Types.ObjectId.isValid(cartId)) {
      return res.status(400).json({
        success: false,
        message: "Invalid cartId. Please provide a valid cart Id.",
      });
    }

    // Find the cart by cartId and populate items with their products and categories
    const cart = await Cart.findById(cartId).populate({
      path: "items",
      populate: {
        path: "productId",
        populate: {
          path: "category subCategory",
          select: "name",
        },
        select: "name price",
      },
    });

    if (!cart) {
      return res
        .status(404)
        .json({ success: false, message: "Cart not found" });
    }

    // Find the patient associated with the cart
    const patient = await Patient.findById(cart.patientId);

    if (!patient) {
      return res
        .status(404)
        .json({ success: false, message: "Patient not found" });
    }

    // Calculate the total amount for each product in the cart
    const products = cart.items.map((item) => {
      const { productId, quantity } = item;
      const product = productId;
      const totalAmount = product.price * quantity;

      return {
        name: product.name,
        quantity,
        price: product.price,
        totalAmount,
        category: product.category.name,
        subCategory: product.subCategory.name,
      };
    });

    // Calculate the grand total of the cart
    const grandTotal = products.reduce(
      (total, product) => total + product.totalAmount,
      0
    );

    res.status(200).json({
      success: true,
      patientName: patient.name,
      cart,
      products,
      grandTotal,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ success: false, message: "Internal server error" });
  }
});

module.exports = router;
